#! /usr/bin/env python
#
# util_RandomizeOverlapOrder.py
#
# 
# GOAL
#   - take an overlap.xml.gz (SimInspiralTable), and randomize its order.  
#   - Important when merging files from many workers, to avoid accidentally using only the output from one of them.  Run after ligolw_add
#   - takes equal number from each file by default ... WARNING THIS MEANS WE CAN HAVE PATHOLOGICAL SIZE LIMITS


import sys
from optparse import OptionParser
import numpy as np


optp = OptionParser()
optp.add_option("--fref",default=20,type=float,help="Reference frequency. Depending on approximant and age of implementation, may be ignored")
optp.add_option("--n-min",default=20,type=int,help="Minimum size of file to include")
optp.add_option("--output-file",default='merged_output',type=str,help="Merged output file")
opts, args = optp.parse_args()

print(" Inputs:  ", '\n\t  '.join(args))
print(" Output: ", opts.output_file)

import os
#os.system(" touch {} ".format(opts.output_file))
os.system(' echo {}  > {}'.format(' '.join(args), opts.output_file))



#! /usr/bin/env python
#
# util_NRRelabelILE.py
#
# GOAL
#    Accepts ILE default .dat output (indx m1 m2 s1x s1y s1z s2x s2y s2z XXX) and converts to group param XXX
#    If option specified, picks a *single* element which has the highest lnL
#
#


import argparse
import sys
import numpy as np
import RIFT.lalsimutils as lalsimutils


from multiprocessing import Pool
try:
    import os
    n_threads = int(os.environ['OMP_NUM_THREADS'])
    print(" Pool size : ", n_threads)
except:
    n_threads=1
    print(" - No multiprocessing - ")

try:
	import NRWaveformCatalogManager3 as nrwf
	hasNR =True
except:
	hasNR=False
try:
    hasEOB=True
    import EOBTidalExternal as eobwf
except:
    hasEOB=False


###
### Load options
###

parser = argparse.ArgumentParser()
parser.add_argument("--group",default=None)
parser.add_argument("--fname", default=None, help="Base output file for ascii text (.dat) and xml (.xml.gz)")
parser.add_argument("--eccentricity",action="store_true",default=False)
parser.add_argument("--verbose", action="store_true",default=False, help="Required to build post-frame-generating sanity-test plots")
opts=  parser.parse_args()

if opts.verbose:
    True
    #lalsimutils.rosDebugMessagesContainer[0]=True   # enable error logging inside lalsimutils

if not opts.fname:
    sys.exit(0)

dat = np.loadtxt(opts.fname)
# check shape, fix if just one line
if len(dat.shape)<2:
    dat = np.array([dat])

best_matches = {}
best_matches_masses ={}
best_matches_xi = {}
for line in dat:
#    print line
    m1 = line[1]
    m2 = line[2]
    s1x,s1y,s1z= line[3:6]
    s2x,s2y,s2z= line[6:9]
    if opts.eccentricity:
        eccentricity = line[9]
    params_to_test = {}
    params_to_test['q'] = m2/m1
    params_to_test['s1x'] = s1x
    params_to_test['s1y'] = s1y
    params_to_test['s1z'] = s1z
    params_to_test['s2x'] = s2x
    params_to_test['s2y'] = s2y
    params_to_test['s2z'] = s2z
    if opts.eccentricity:
        params_to_test['eccentricity'] = eccentricity
    if opts.group:
        matches = nrwf.NRSimulationLookup(params_to_test,valid_groups=[opts.group])
    else:
        matches = nrwf.NRSimulationLookup(params_to_test)
    if not len(matches)>0:
        pass
    # Pick the longest simulation"
    try:
      if len(matches)> 2:
            
        print("   Attempting to pick the longest simulation matching  the simulation from ", matches)
        MOmega0  = 1
        tmax = -1
        good_sim = None
        for key in matches:
#            print "    ... trying ", key
            # First choice is to look up by Momega0
            if 'Momega0' in nrwf.internal_WaveformMetadata[key[0]][key[1]]  and  tmax == -1:
                if nrwf.internal_WaveformMetadata[key[0]][key[1]]['Momega0'] < MOmega0:
                    good_sim = key
                    MOmega0 = nrwf.internal_WaveformMetadata[key[0]][key[1]]['Momega0']
            # second choice is to look up by NR peak value.  This may not be available (i.e., no preprocessed data)  , so the entire system could crash
            else:
                if nrwf.internal_EstimatePeakL2M2Emission[key[0]][key[1]] > tmax:
                    good_sim = key
                    tmax = nrwf.internal_EstimatePeakL2M2Emission[key[0]][key[1]]
#        print " Picked  ",key,  " with MOmega0 ", MOmega0, " and peak duration ", nrwf.internal_EstimatePeakL2M2Emission[key[0]][key[1]]
        matches = [good_sim]
    except:
        matches  = [matches[0]] # pick the first one.  Note we will want to reduce /downselect the lookup process

    if len(matches)>0:
        lnL_indx = 9
        if opts.eccentricity:
            lnL_indx = 10
        if opts.verbose:
            print(matches[0][0], matches[0][1], line[lnL_indx], line[lnL_indx+1], line[lnL_indx+2], line[lnL_indx+3])
        if matches[0] in best_matches:
            if best_matches[matches[0]] < line[lnL_indx]:
#                print " Replacing "
                best_matches[matches[0]] = line[lnL_indx]
                best_matches_masses[matches[0]] = (line[1],line[2])
                q = m2/m1
                best_matches_xi[matches[0]] = (s1z + q*s2z)/(1+q)  # assumes simulation L is parallel to z
        else:
            best_matches[matches[0]] = line[lnL_indx]
            best_matches_masses[matches[0]] = (line[1],line[2])
            q = m2/m1
            best_matches_xi[matches[0]] = (s1z + q*s2z)/(1+q)  # assumes simulation L is parallel to z

print(" -----  BEST MATCHES ------ ")  # unsorted
for key in best_matches:
    tmax =0
    if key[1] in nrwf.internal_EstimatePeakL2M2Emission[key[0]]:
        tmax = nrwf.internal_EstimatePeakL2M2Emission[key[0]][key[1]]
    af = -1000
    Mf = -1000
    if 'ChiFMagnitude' in nrwf.internal_WaveformMetadata[key[0]][key[1]]:
        af = nrwf.internal_WaveformMetadata[key[0]][key[1]]['ChiFMagnitude']
    if 'MF' in nrwf.internal_WaveformMetadata[key[0]][key[1]]:
        Mf = nrwf.internal_WaveformMetadata[key[0]][key[1]]['MF']
    print(best_matches[key], key,   best_matches_masses[key][0], best_matches_masses[key][1], tmax, best_matches_xi[key], Mf, af)



#
# gwsignal wrapper
#  
#  WHY IS THIS NOT IN lalsimutils
#     - gwsignal (via gwsurrogate) prints to stdout some stupid messages, which breaks several scripts which write to stdout
#     - gwsignal is a slow import with a lot of large dependencies, not well suited to our lowlatency use.


# References:
#   https://git.ligo.org/waveforms/new-waveforms-interface/-/blob/master/python_interface/docs/source/examples/example_usage.ipynb

import lal
import lalsimulation as lalsim
import RIFT.lalsimutils as lalsimutils
import numpy as np
import astropy.units as u
from astropy.time import Time
from gwpy.timeseries import TimeSeries
import astropy.constants as ac
has_gws= False
try:
    # Warning: prints stupid messages to stdout
    try: # if hasattr(lalsim, 'gwsignal'):
        import lalsimulation.gwsignal as gws
        from lalsimulation.gwsignal.core import utils as ut
        from lalsimulation.gwsignal.core import waveform as wfm
    except: # else:
        import gwsignal as gws
        from gwsignal.core import utils as ut
        from gwsignal.core import waveform as wfm
    has_gws=True
except:
    has_gws=False
    print("GWsignal import failed")


def std_and_conj_hlmoff(P, Lmax=2,approx_string=None,**kwargs):
    hlms = hlmoft(P, Lmax,approx_string=approx_string,**kwargs)
    hlmsF = {}
    hlms_conj_F = {}
    for mode in hlms:
        hlmsF[mode] = lalsimutils.DataFourier(hlms[mode])
        hlms[mode].data.data = np.conj(hlms[mode].data.data)
        hlms_conj_F[mode] = lalsimutils.DataFourier(hlms[mode])
    return hlmsF, hlms_conj_F

def hlmoff(P, Lmax=2,approx_string=None,**kwargs):
    hlms = hlmoft(P, Lmax,approx_string=approx_string,**kwargs)
    hlmsF = {}
    for mode in hlms:
        hlmsF[mode] = lalsimutils.DataFourier(hlms[mode])
    return hlmsF


def hlmoft(P, Lmax=2,approx_string=None,no_trust_align_method=None,**kwargs):
    """
    gwsignal.  Note the call will use approx_string, NOT a lalsimulation mode ID.  If approx_string is none, use P.approx but convert to string
    """

    assert Lmax >= 2

    # Check that masses are not nan!
    assert (not np.isnan(P.m1)) and (not np.isnan(P.m2)), " masses are NaN "
    taper=0
    if P.taper != lalsim.SIM_INSPIRAL_TAPER_NONE:
        taper = 1
    python_dict = {'mass1' : P.m1/lal.MSUN_SI * u.solMass,
              'mass2' : P.m2/lal.MSUN_SI * u.solMass,
              'spin1x' : P.s1x*u.dimensionless_unscaled,
              'spin1y' : P.s1y*u.dimensionless_unscaled,
              'spin1z' : P.s1z*u.dimensionless_unscaled,
              'spin2x' : P.s2x*u.dimensionless_unscaled,
              'spin2y' : P.s2y*u.dimensionless_unscaled,
              'spin2z' : P.s2z*u.dimensionless_unscaled,
              'deltaT' : P.deltaT*u.s,
              'f22_start' : P.fmin*u.Hz,
              'f22_ref': P.fref*u.Hz,
              'phi_ref' : P.phiref*u.rad,
              'distance' : P.dist/(1e6*lal.PC_SI)*u.Mpc,
              'inclination' : P.incl*u.rad,
              'eccentricity' : P.eccentricity*u.dimensionless_unscaled,
              'longAscNodes' : P.psi*u.rad,
              'meanPerAno' : P.meanPerAno*u.rad,
              'condition' : taper     }
    if 'lmax_nyquist' in kwargs:
        python_dict['lmax_nyquist'] = kwargs['lmax_nyquist']

    # if needed
#    lal_dict = gws.core.utils.to_lal_dict(python_dict)

    approx_string_here = approx_string
    if not(approx_string):
        approx_string_here = lalsim.GetStringFromApproximant(P.approx)

    # Fork on calling different generators
    gen = gws.models.gwsignal_get_waveform_generator(approx_string_here)
    # if "NRSur7dq4_gwsurr" == approx_string_here:
    #     gen =gws.NRSur7dq4_gwsurr()
    # elif approx_string_here == 'SEOBNRv5PHM':  # only available
    #     gen = gws.models.pyseobnr.SEOBNRv5PHM()
    # else:
    #     gen = wfm.LALCompactBinaryCoalescenceGenerator(approx_string_here)

    hlm = wfm.GenerateTDModes(python_dict,gen)
    tvals = hlm[(2,2)].times
    npts = len(tvals)
    epoch = float(tvals[0]/u.second)

    # Repack in conventional structure (typing)
    hlmT = {}
    for mode in hlm:
        if isinstance(mode, str):  # skip 'time_array'
            continue
        if mode[0] > Lmax:  # skip modes with L > Lmax
            continue
        # 
        h = lal.CreateCOMPLEX16TimeSeries("hlm",
                lal.LIGOTimeGPS(0.), 0., P.deltaT, lal.DimensionlessUnit,
                npts)
        h.data.data = np.array(hlm[mode])
        h.epoch = epoch
        # Resize if needed
        if P.deltaF:
            TDlen = int(1./P.deltaF * 1./P.deltaT)
            if TDlen < h.data.length:   # Truncate the series to the desired length, removing data at the *start* (left)
                h = lal.ResizeCOMPLEX16TimeSeries(h,h.data.length-TDlen,TDlen)
            elif TDlen > h.data.length:   # Zero pad, extend at end
                h = lal.ResizeCOMPLEX16TimeSeries(h,0,TDlen)
        # WARNING:  realistically, the GWSignal mode output was NEVER tapered, oddly -- so do it by hand, following lalsimutils choices
        if taper:
            ntaper = int(0.01*TDlen)
            if P.fmin > 0: # avoid failure if waveform start frequency 0 is nominally specified
                ntaper = np.max([ntaper, int(1./(P.fmin*P.deltaT))]) 
            vectaper= 0.5 - 0.5*np.cos(np.pi*np.arange(ntaper)/(1.*ntaper))
            # Taper at the start of the segment
            h.data.data[:ntaper]*=vectaper
        # Add to structure
        hlmT[mode] = h

    # if no_trust_peak_method, we will change the epoch.  Standard option is 'peak', to find the peak value of |h|^2, summed over modes
    # Note there is *no interpolation* between samples, so the sampling rate will introduce some jitter.
    if no_trust_align_method == 'peak':
        rhosq = np.zeros(TDlen)
        for mode in hlmT:
            rhosq += np.abs(hlmT.data.data)**2
        indx_max =np.argmax(rhosq)
        new_epoch = - indx_max*P.deltaT
        for mode in hlmT:
            hlmT.epoch = new_epoch
    if approx_string_here == 'TEOBResumSDALI':
        nu = P.m1*P.m2/((P.m1+P.m2)**2)
        distance_rescaling = (
            (
                nu
                * (P.m1 + P.m2)
                / P.dist
                * ac.G
                / ac.c ** 2
            )
            .value
        )
        for mode in hlmT:
            hlmT[mode].data.data = distance_rescaling*hlmT[mode].data.data
            
    return hlmT



#
# Functions to generate waveforms
#
def hoft(P, Fp=None, Fc=None,approx_string=None, **kwargs):
    """
    Generate a TD waveform from ChooseWaveformParams P
    Based on https://git.ligo.org/waveforms/reviews/newwfinterface/-/blob/main/example_usage/example_usage_using_gwsignal_in_lalsimulation.ipynb
    You may pass in antenna patterns Fp, Fc. If none are provided, they will
    be computed from the information in ChooseWaveformParams.

    Returns a REAL8TimeSeries object
    """

    # special sauce for EOB, because it is so finicky regarding
    if P.approx == lalsim.EOBNRv2HM and P.m1 == P.m2:
#        print " Using ridiculous tweak for equal-mass line EOB"
        P.m2 = P.m1*(1-1e-6)
    extra_params = P.to_lal_dict()


    assert (not np.isnan(P.m1)) and (not np.isnan(P.m2)), " masses are NaN "
    taper=0
    if P.taper != lalsim.SIM_INSPIRAL_TAPER_NONE:
        taper = 1
    python_dict = {'mass1' : P.m1/lal.MSUN_SI * u.solMass,
              'mass2' : P.m2/lal.MSUN_SI * u.solMass,
              'spin1x' : P.s1x*u.dimensionless_unscaled,
              'spin1y' : P.s1y*u.dimensionless_unscaled,
              'spin1z' : P.s1z*u.dimensionless_unscaled,
              'spin2x' : P.s2x*u.dimensionless_unscaled,
              'spin2y' : P.s2y*u.dimensionless_unscaled,
              'spin2z' : P.s2z*u.dimensionless_unscaled,
              'deltaT' : P.deltaT*u.s,
              'f22_start' : P.fmin*u.Hz,
              'f22_ref': P.fref*u.Hz,
              'phi_ref' : P.phiref*u.rad,
              'distance' : P.dist/(1e6*lal.PC_SI)*u.Mpc,
              'inclination' : P.incl*u.rad,
              'eccentricity' : P.eccentricity*u.dimensionless_unscaled,
              'longAscNodes' : P.psi*u.rad,
              'meanPerAno' : P.meanPerAno*u.rad,
              'condition' : taper}
    if 'lmax_nyquist' in kwargs:
        python_dict['lmax_nyquist'] = kwargs['lmax_nyquist']

    # if needed
#    lal_dict = gws.core.utils.to_lal_dict(python_dict)

    approx_string_here = approx_string
    if not(approx_string):
        approx_string_here = lalsim.GetStringFromApproximant(P.approx)

    # Fork on calling different generators
    gen = gws.models.gwsignal_get_waveform_generator(approx_string_here)

    # gwsignal return values are sometimes gwsignal objects
    hp, hc = gws.core.waveform.GenerateTDWaveform(python_dict, gen)
    if not isinstance(hp, lal.REAL8TimeSeries):
        # gwpy.timeseries.timeseries.TimeSeries object
        hp_lal = lal.CreateREAL8TimeSeries("hp",
                lal.LIGOTimeGPS(0.), 0., P.deltaT, lal.DimensionlessUnit,
                len(hp.times))
        hc_lal = lal.CreateREAL8TimeSeries("hp",
                lal.LIGOTimeGPS(0.), 0., P.deltaT, lal.DimensionlessUnit,
                len(hc.times))
        hp_lal.data.data =hp.value
        hc_lal.data.data = hc.value
        if isinstance(hp.epoch, Time):
            dT = hp.epoch.to_value('gps','long')  # pull out the time
        else:
            dT = float(hp.epoch) # old-style
        hp_lal.epoch = dT
        hc_lal.epoch = dT
        hp = hp_lal
        hc = hc_lal

    if Fp!=None and Fc!=None:
        hp.data.data *= Fp
        hc.data.data *= Fc
        hp = lal.AddREAL8TimeSeries(hp, hc)
        ht = hp
    elif P.radec==False:
        fp = Fplus(P.theta, P.phi, P.psi)
        fc = Fcross(P.theta, P.phi, P.psi)
        hp.data.data *= fp
        hc.data.data *= fc
        hp = lal.AddREAL8TimeSeries(hp, hc)
        ht = hp
    else:
        # If astropy Time function, overwrite with GPS time, otherwise use normal addition
        if isinstance(hp.epoch, Time):
            dT = hp.epoch.to_value('gps','long')  # pull out the time
            hp.epoch = P.tref + dT
            hc.epoch = P.tref +dT
        else:
            hp.epoch = hp.epoch + P.tref
            hc.epoch = hc.epoch + P.tref
        ht = lalsim.SimDetectorStrainREAL8TimeSeries(hp, hc, 
                P.phi, P.theta, P.psi, 
                lalsim.DetectorPrefixToLALDetector(str(P.detector)))
    if P.taper != lalsimutils.lsu_TAPER_NONE: # Taper if requested
        lalsim.SimInspiralREAL8WaveTaper(ht.data, P.taper)
    if P.deltaF is not None:
        TDlen = int(1./P.deltaF * 1./P.deltaT)
        assert TDlen >= ht.data.length
        ht = lal.ResizeREAL8TimeSeries(ht, 0, TDlen)
    return ht


def complex_hoft(P, Fp=None, Fc=None,approx_string=None,sgn=-1, **kwargs):
    """
    Similarly. 
    """

    # special sauce for EOB, because it is so finicky regarding
    if P.approx == lalsim.EOBNRv2HM and P.m1 == P.m2:
#        print " Using ridiculous tweak for equal-mass line EOB"
        P.m2 = P.m1*(1-1e-6)
    extra_params = P.to_lal_dict()


    assert (not np.isnan(P.m1)) and (not np.isnan(P.m2)), " masses are NaN "
    taper=0
    if P.taper != lalsim.SIM_INSPIRAL_TAPER_NONE:
        taper = 1
    python_dict = {'mass1' : P.m1/lal.MSUN_SI * u.solMass,
              'mass2' : P.m2/lal.MSUN_SI * u.solMass,
              'spin1x' : P.s1x*u.dimensionless_unscaled,
              'spin1y' : P.s1y*u.dimensionless_unscaled,
              'spin1z' : P.s1z*u.dimensionless_unscaled,
              'spin2x' : P.s2x*u.dimensionless_unscaled,
              'spin2y' : P.s2y*u.dimensionless_unscaled,
              'spin2z' : P.s2z*u.dimensionless_unscaled,
              'deltaT' : P.deltaT*u.s,
              'f22_start' : P.fmin*u.Hz,
              'f22_ref': P.fref*u.Hz,
              'phi_ref' : P.phiref*u.rad,
              'distance' : P.dist/(1e6*lal.PC_SI)*u.Mpc,
              'inclination' : P.incl*u.rad,
              'eccentricity' : P.eccentricity*u.dimensionless_unscaled,
              'longAscNodes' : P.psi*u.rad,
              'meanPerAno' : P.meanPerAno*u.rad,
              'condition' : taper}
    if 'lmax_nyquist' in kwargs:
        python_dict['lmax_nyquist'] = kwargs['lmax_nyquist']

    # if needed
#    lal_dict = gws.core.utils.to_lal_dict(python_dict)

    approx_string_here = approx_string
    if not(approx_string):
        approx_string_here = lalsim.GetStringFromApproximant(P.approx)

    # Fork on calling different generators
    gen = gws.models.gwsignal_get_waveform_generator(approx_string_here)

    # gwsignal return values are sometimes gwsignal objects
    hp, hc = gws.core.waveform.GenerateTDWaveform(python_dict, gen)
    if not isinstance(hp, lal.REAL8TimeSeries):
        # gwpy.timeseries.timeseries.TimeSeries object
        hp_lal = lal.CreateREAL8TimeSeries("hp",
                lal.LIGOTimeGPS(0.), 0., P.deltaT, lal.DimensionlessUnit,
                len(hp.times))
        hc_lal = lal.CreateREAL8TimeSeries("hp",
                lal.LIGOTimeGPS(0.), 0., P.deltaT, lal.DimensionlessUnit,
                len(hc.times))
        hp_lal.data.data =hp.value
        hc_lal.data.data = hc.value
        if isinstance(hp.epoch, Time):
            dT = hp.epoch.to_value('gps','long')  # pull out the time
        else:
            dT = float(hp.epoch) # old-style
        hp_lal.epoch = dT
        hc_lal.epoch = dT
        hp = hp_lal
        hc = hc_lal


    ht = lal.CreateCOMPLEX16TimeSeries("Complex h(t)", hp.epoch, hp.f0, 
                                       hp.deltaT, lalsimutils.lsu_DimensionlessUnit, hp.data.length)
    ht.epoch = ht.epoch + P.tref
    ht.data.data = hp.data.data + 1j * sgn * hc.data.data
    # impose polarization directly, using precisely the conventions we demand
    ht.data.data*= np.exp(2j*sgn*P.psi)
    return ht



#!/usr/bin/python
# Monica Rizzo

#  python ~/research-projects/MonteCarloMarginalizeCode/Code/util_GenerateMaxlnLWaveform_NRFromIndex.py --nr-group Sequence-SXS-All --nr-param 2 --run-dir G184098-v_test_hybrid-SingleSimNR-1-Sequence-SXS-All-Lmax2-fmin-10 --fname-indexed G184098-v_test_hybrid-SingleSimNR-1-Sequence-SXS-All-Lmax2-fmin-10_wrapup.indexed --verbose
#
# FIXME
#   - SXS: Switch to use-provided-strain, if it was used
#   - if maxpt not available, run ILE in single-point mode to find best fit parameters, adding --maximize-only
#   - omit memory

import numpy as np
import sys
import argparse
import os
import subprocess
from shutil import copyfile

import NRWaveformCatalogManager3 as nrwf
import lal
import RIFT.lalsimutils as lalsimutils

parser = argparse.ArgumentParser()
parser.add_argument("--l-max", type=int, default=2, help="Include all (l,m) modes with l less than or equal to this value.")
parser.add_argument("--run-dir",default=None, help="directory code was run in")
parser.add_argument("--nr-group",default=None,help="group")
parser.add_argument("--nr-param",default=None,help="param")
parser.add_argument("--fname-indexed",default=None,help="File name of *.indexed file")
parser.add_argument("--n-max",default=1e4,type=int,help="Override settings in command-single.sh")
parser.add_argument("--event-time", type=np.float128,  default=None,help="Event time. If not present, will use event.log")
parser.add_argument("--save-plots", default=False, action='store_true',help="saves waveform plot and hoft data")
parser.add_argument("--use-NR", default=False, action='store_true', help="generate plots using NR data")
parser.add_argument("--no-memory",action='store_true')
parser.add_argument("--sigma-cut",default=0.8,type=float)
parser.add_argument("--M-max-cut",default=200,type=float)
parser.add_argument("--verbose",action='store_true')
opts=parser.parse_args()

Lmax = opts.l_max


#look through run directory for file containing max lnl

#arrays to store lnl values and file numbers
lnl_vals = []
fnums = []

if not opts.fname_indexed:
    sys.exit(0)


#util_dir = "/home/oshaughn/research-projects/MonteCarloMarginalizeCode/Code/"

###
### # Parse  *.indexed (see util_NRBestOfIndex.py)
###
best_matches = {}
best_matches_masses ={}
full_spoke = {}
with open(opts.fname_indexed) as f:
 for line_str in f:
     # string split
     line = line_str.split()
     if opts.verbose:
         print("  Input ", line)
     # Get NR data
     if float(line[1])+float(line[2]) > opts.M_max_cut:
         continue
     group = line[3]
     if not (group == opts.nr_group):
         if opts.verbose:
             print(" Bad group ", group)
         continue
     if not nrwf.internal_ParametersAreExpressions.has_key(group):
         continue
     param = None
     if nrwf.internal_ParametersAreExpressions[group]:
         param = eval(line[4])
     else:
         param = line[4]

     if not (param == opts.nr_param):
         if opts.verbose:
             print(" Bad param", param)    # Problem: this can happen because of the lookup code! Equivalent physical parameters
         continue

     if len(line)<6:
         continue

     key = (group,param)

     failure_mode=False; sigma_here = 1e6
     # in case line abbreviated for some reason
     try:
         lnLhere = float(line[5])  
         sigma_here = float(line[6])
         npts_here = float(line[7])
     except:
         lnLhere = -10
         sigma_here = 0.1
         npts_here = 1
         failure_mode=True
         continue
     if sigma_here > opts.sigma_cut:
         failure_mode=True
         continue  # DO NOT RECORD ITEMS which are completely unconverged (one point). Insane answers likely.  (should ALSO have tunable cutoff on accuracy)
     if not failure_mode:
      if best_matches.has_key(key):
         if best_matches[key] < lnLhere:
                best_matches[key] = lnLhere
                best_matches_masses[key] = (float(line[1]),float(line[2]))
      else:
         best_matches[key] = line[5]
         best_matches_masses[key] = (float(line[1]),float(line[2]))

      # Add to spoke
      mtot = float(line[1])+ float(line[2])
      if full_spoke.has_key(key):
        full_spoke[key].append([mtot, lnLhere,sigma_here])
      else:
         full_spoke[key] = [[mtot,lnLhere, sigma_here]]




###
### Create ILE job to follow it up
###

# Create xml file with correct mass and NR parameters 
m1,m2 = best_matches_masses[(opts.nr_group,opts.nr_param)]
wfP =  nrwf.WaveformModeCatalog(opts.nr_group, opts.nr_param, metadata_only=True)
wfP.P.assign_param('mtot',(m1+m2)*lal.MSUN_SI)
wfP.P.print_params()
P_list = [wfP.P]
lalsimutils.ChooseWaveformParams_array_to_xml(P_list, fname="single-pt")  # choice of fref irrelevant




        

with open(str(opts.run_dir)+"/command-single.sh",'r') as runfile:
    rf=str(runfile.readlines()[1])
    rf=rf.replace('create_event_dag_via_grid', 'integrate_likelihood_extrinsic')
    rf+=" --maximize-only"
    rf=rf.split()
    rf[rf.index("--sim-xml")+1]="single-pt.xml.gz"
    rf[rf.index("--output-file")+1]="ILE-single.xml.gz"  
    if "--n-copies" in rf:
        rf[rf.index("--n-copies")+1]=""
    if "--n-max" in rf:
        rf[rf.index("--n-max")+1]=" 10000 "
    rf_submit = ' '.join(rf)
    if "--n-copies" in rf:
        rf_submit=rf_submit.replace("--n-copies","")
        
    print(rf_submit)
    os.system(rf_submit)
       


    #acquire event time
    if opts.event_time == None:
        with open(str(opts.run_dir)+"/event.log", 'r') as log:
            for line in log:
                if "Time" in line: #line.startswith("End"):
                    event_time=line.split()[-1]
    else:
        event_time = opts.event_time
    
    infile = 'maxpt_ILE-single.xml.gz.xml.gz'

    cmd ="util_NRDumpDetectorResponse.py --inj "+infile+" --event 0 --t-ref "+str(event_time)+" --group "+opts.nr_group+' --param  '+str(opts.nr_param).replace(' ', '')+' --l ' + str(opts.l_max) + " --use-perturbative-extraction" 
    if opts.no_memory:
        cmd = cmd+ " --no-memory "
    
    if opts.save_plots:
        cmd+=" --save-plots --verbose"

os.chdir(opts.run_dir)
print(cmd)
os.system(cmd)




import sys
import functools
from optparse import OptionParser

import numpy as np

import lal
import lalsimulation as lalsim
from glue.ligolw import utils, lsctables, table
try:
    from matplotlib import pylab as plt
except:
    print(" No interactive plots for you ! ")

import lalsimutils
import factored_likelihood

__author__ = "R. O'Shaughnessy"

rosUseWindowing = True

#
# Option parsing
#

optp = OptionParser()
optp.add_option("-p", "--psd-file", default="psd.xml.gz",help="instrument=psd-file, e.g. H1=H1_PSD.xml.gz. Can be given multiple times for different instruments.")
opts, args = optp.parse_args()

print(opts, args)
print(opts.psd_file)
#
# Load in PSDs: original API
#
# psdf = opts.psd_file
# psd_dict ={}
# psd_dict['H1'] = lalsimutils.get_psd_series_from_xmldoc(psdf, 'H1')
# psd_dict['L1'] = lalsimutils.get_psd_series_from_xmldoc(psdf, 'L1')
# psd_dict['V1'] = lalsimutils.get_psd_series_from_xmldoc(psdf, 'V1')
# fvals = psd_dict['H1'] .deltaF*np.arange(len(psd_dict['H1'].data))

#
# Load in PSDs: convert to swig first
#
psdf = opts.psd_file
psd_dict ={}
psd_dict['H1'] =lalsimutils.pylal_psd_to_swig_psd( lalsimutils.get_psd_series_from_xmldoc(psdf, 'H1'))
psd_dict['L1'] = lalsimutils.pylal_psd_to_swig_psd( lalsimutils.get_psd_series_from_xmldoc(psdf, 'L1'))
psd_dict['V1'] = lalsimutils.pylal_psd_to_swig_psd( lalsimutils.get_psd_series_from_xmldoc(psdf, 'V1'))
fvals = psd_dict['H1'] .deltaF*np.arange(len(psd_dict['H1'].data.data))


# Report on PSD -- in particular, on the nyquist bin
detectors = psd_dict.keys()
print("  === 'Nyquist bin' report  ==== ")
for det in detectors:
    df = psd_dict[det].deltaF
    print(det, " has length ", len(psd_dict[det].data.data), " with nyquist bin value ",  psd_dict[det].data.data[-1], " which had better be EXACTLY zero if we don't explicitly excise it.  Note also  second-to-last bin, etc are also low:  ", psd_dict[det].data.data[-2], psd_dict[det].data.data[-int(10/df)],psd_dict[det].data.data[-int(30/df)])
print(" ... so several bins may be anomalously small ... ")
for det in psd_dict.keys():
    pairups = np.transpose(np.array([fvals,psd_dict[det].data.data]))
    badbins = [k[0] for k in pairups if k[0]>100 and  k[1]<100*psd_dict[det].data.data[-1]]
    print(" highest reliable frequency in ", det, " seems to be ", badbins[0])

# Plot the raw  PSD
print(" === Plotting === ")
for det in detectors:
    if rosUseWindowing:
        tmp =  lalsimutils.regularize_swig_psd_series_near_nyquist(psd_dict[det], 80) # zero out 80 hz window near nyquist
        psd_dict[det] =  lalsimutils.enforce_swig_psd_fmin(tmp, 30.)
    plt.loglog(fvals, psd_dict[det].data.data,label='psd:'+det)
fn = np.frompyfunc(lambda x: lalsim.SimNoisePSDaLIGOZeroDetHighPower(x) if x>30 else 0,1,1)
#psd_guess = np.log10(np.array(fn(fvals)))
psd_guess = np.zeros(len(fvals))
psd_guess[1:-1] = np.array(map(lalsim.SimNoisePSDaLIGOZeroDetHighPower,fvals[1:-1]))
plt.loglog(fvals,psd_guess,label='analytic')
plt.legend()
#plt.xlim(1e1,1e2)
plt.show()

#
# 'Interpolate' PSD and replot  [option to use the 'regularize']
#
print(" === Interpolating and replotting === ")
psd_extend = {}
plt.clf()
for det in detectors:
    df = psd_dict[det].deltaF
    psd_extend[det] = lalsimutils.extend_swig_psd_series_to_sampling_requirements(psd_dict[det], df/2, (len(psd_dict[det].data.data)-1)*df)
    fvals2 = df/2*np.arange(len(psd_extend[det].data.data))
    plt.loglog(fvals2, psd_extend[det].data.data,label=det)
plt.legend()
plt.show()

#
# Generate an inner product using the original discrete PSD
# Perform an inner product using it *and* using an analytic PSD
#
# Populate signal
m1 = 10*lal.LAL_MSUN_SI
m2 = 10*lal.LAL_MSUN_SI

df = psd_dict[detectors[0]].deltaF
fSample = df * 2 *( len(psd_dict[detectors[0]].data.data)-1)  # rescale
print("To construct a signal, we  reconstruct the sampling rate and time window, consistent with the default PSD sampling: (fSample, 1/df) = ", fSample, 1/df)
Psig = lalsimutils.ChooseWaveformParams(
    m1 = m1,m2 =m2,
    fmin = 30, 
    fref=100, ampO=0,
    tref = lal.GPSTimeNow(),   # factored_likelihood requires GPS be assigned 
    radec=True, theta=1.2, phi=2.4,
    detector='H1', 
    dist=25.*1.e6*lal.LAL_PC_SI,
    deltaT=1./fSample,
    deltaF = df
    )
data_dict={}
data_dict['H1'] = lalsimutils.non_herm_hoff(Psig)
Psig.detector = 'L1'
data_dict['L1'] = lalsimutils.non_herm_hoff(Psig)
Psig.detector = 'V1'
data_dict['V1'] = lalsimutils.non_herm_hoff(Psig)
psd_analytic_dict = {}
psd_analytic_dict['H1'] = lalsim.SimNoisePSDaLIGOZeroDetHighPower# lal.LIGOIPsd
psd_analytic_dict['L1'] = lalsim.SimNoisePSDaLIGOZeroDetHighPower #lal.LIGOIPsd
psd_analytic_dict['V1'] = lalsim.SimNoisePSDaLIGOZeroDetHighPower # lal.LIGOIPsd


for det in psd_dict.keys():
    fNyq = (len(psd_dict[det].data.data)-1)*psd_dict[det].deltaF
    print(fNyq)
    print(" Length consistency requirements : ", 2*(len(psd_dict[det].data.data)-1), 2*fNyq/df  -1 , len(data_dict[det].data.data))
    IP = lalsimutils.ComplexIP(fLow=30, fNyq=fNyq,deltaF=df,psd=psd_dict[det].data.data,analyticPSD_Q=False)
    IPAnalytic = lalsimutils.ComplexIP(fLow=30, fNyq=fNyq,deltaF=df,psd=psd_analytic_dict[det],analyticPSD_Q=True)

    rho1 = IP.norm(data_dict[det])
    rho2 = IPAnalytic.norm(data_dict[det])

    print(det, rho1, rho2)

#
# Interpolate PSD and repeat the above test
# 
#    psd_dict[inst] = lalsimutils.extend_psd_series_to_sampling_requirements(psd_dict[inst], deltaF, deltaF*len(data_dict[inst].data.data))
#
# Populate signal
df = psd_extend[detectors[0]].deltaF
fSample = df * 2 *( len(psd_extend[detectors[0]].data.data)-1)  # rescale
print("To construct a signal, we  reconstruct the sampling rate and time window, consistent with the default PSD sampling: (fSample, 1/df) = ", fSample, 1/df)
Psig = lalsimutils.ChooseWaveformParams(
    m1 = m1,m2 =m2,
    fmin = 30, 
    fref=100, ampO=0,
    tref = lal.GPSTimeNow(),   # factored_likelihood requires GPS be assigned 
    radec=True, theta=1.2, phi=2.4,
    detector='H1', 
    dist=25.*1.e6*lal.LAL_PC_SI,
    deltaT=1./fSample,
    deltaF = df
    )
data_dict={}
data_dict['H1'] = factored_likelihood.non_herm_hoff(Psig)
Psig.detector = 'L1'
data_dict['L1'] = factored_likelihood.non_herm_hoff(Psig)
Psig.detector = 'V1'
data_dict['V1'] = factored_likelihood.non_herm_hoff(Psig)
psd_analytic_dict = {}
psd_analytic_dict['H1'] = lalsim.SimNoisePSDaLIGOZeroDetHighPower# lal.LIGOIPsd
psd_analytic_dict['L1'] = lalsim.SimNoisePSDaLIGOZeroDetHighPower #lal.LIGOIPsd
psd_analytic_dict['V1'] = lalsim.SimNoisePSDaLIGOZeroDetHighPower # lal.LIGOIPsd


for det in psd_extend.keys():
    fNyq = (len(psd_extend[det].data.data)-1)*psd_extend[det].deltaF
    print(fNyq)
    print(" Length consistency requirements : ", 2*(len(psd_extend[det].data.data)-1), 2*fNyq/df  -1 , len(data_dict[det].data.data))
    IP = lalsimutils.ComplexIP(fLow=30, fNyq=fNyq,deltaF=df,psd=psd_extend[det].data.data,analyticPSD_Q=False)
    IPAnalytic = lalsimutils.ComplexIP(fLow=30, fNyq=fNyq,deltaF=df,psd=psd_analytic_dict[det],analyticPSD_Q=True)

    rho1 = IP.norm(data_dict[det])
    rho2 = IPAnalytic.norm(data_dict[det])

    print(det, rho1, rho2)





#! /usr/bin/env python

import RIFT.lalsimutils as lalsimutils
import numpy as np

chi_max = 0.5
npts=2000
dat_all = np.empty((npts,13))
for i in np.arange(npts):
    m1  =np.random.uniform(50,60)
    m2  =np.random.uniform(0.8*m1,m1)
    P =lalsimutils.ChooseWaveformParams()
    P.randomize()
    s1x = chi_max * P.s1x
    s1y = chi_max * P.s1y
    s1z = chi_max * P.s1z
    s2x = chi_max * P.s2x
    s2y = chi_max * P.s2y
    s2z = chi_max * P.s2z

    line =[-1 , m1,m2,s1x, s1y,s1z,s2x, s2y,s2z, 100, 0.1, 100, -1]
    dat_all[i,:] = np.array(line)

np.savetxt("fake.composite",dat_all,header=" indx m1 m2 s1x s1y s1z s2x s2y s2z lnL sigmaL neff npts")



#!/usr/bin/env python
#  util_CoincIFOUtility.py
#
#  Usage 1: Print IFOs taking part in a coinc
#  Usage 2: Construct command-line arguments needed for data selection in ILE
#  Usage 3: Construct data taking needed for doubles (unnecessary)



import RIFT.misc.ourparams as ourparams
import numpy as np
opts,  rosDebugMessagesDictionary = ourparams.ParseStandardArguments()

from igwn_ligolw import utils, lsctables, table, ligolw

if opts.coinc:
    xmldoc = utils.load_filename(opts.coinc)
    coinc_table = lsctables.CoincInspiralTable.get_table(xmldoc)
    assert len(coinc_table) == 1
    coinc_row = coinc_table[0]
       # Populate the SNR sequence and mass sequence
    sngl_inspiral_table = table.get_table(xmldoc, lsctables.SnglInspiralTable.tableName)
    detList = []
    for sngl_row in sngl_inspiral_table:
        detList.append(str(sngl_row.ifo))

print(detList)



#!/usr/bin/env python

from __future__ import print_function

import os.path
import pkgutil
import re
import sys
from importlib import import_module

import pytest

# Set environment variables as needed
import os
if not('GW_SURROGATE' in os.environ.keys()):
   os.environ["GW_SURROGATE"]=''
# EOBTidalExternal - no one should ever use this code
if not('EOB_BASE' in os.environ.keys()):
   os.environ["EOB_BASE"]=''
if not('EOB_ARCHIVE' in os.environ.keys()):
   os.environ["EOB_ARCHIVE"]='~/'
if not('MATLAB_BASE' in os.environ.keys()):
   os.environ["MATLAB_BASE"]=''
# EOBTidalExternalC - probably ditto
if not('EOB_C_BASE' in os.environ.keys()):
   os.environ["EOB_C_BASE"]=''
if not('EOB_C_ARCHIVE' in os.environ.keys()):
   os.environ["EOB_C_ARCHIVE"]='~/'
if not('EOB_C_ARCHIVE_NMAX' in os.environ.keys()):
   os.environ["EOB_C_ARCHIVE_NMAX"]='1'
# EOSManager
if not('EOS_TABLES' in os.environ.keys()):
   os.environ["EOS_TABLES"]='~/'
# lalsuite
if not('LALSIMULATION_DATADIR' in os.environ.keys()):
   os.environ["LALSIMULATION_DATADIR"]='~/'



pkgname = "RIFT"
package = import_module(pkgname)

# files to ignore
EXCLUDE = re.compile("({})".format("|".join([
    r"\Atests\Z",
    r"\Atest_",
    r"\Aconftest\Z",
    r"\A_",
    r"\Aasimov\Z",
])))

# ignorable failures
IGNORE = re.compile("({})".format("|".join([
    r"\ANo module named torch\Z",
    r"\ANo module named 'torch'\Z",
    r"\ANo module named cupy\Z",
    r"\ANo module named 'cupy'\Z",
    r"\ANo module named asimov\Z",
    r"\ANo module named 'asimov'\Z",
])))


def iter_all_modules(path, exclude=EXCLUDE):
    name = os.path.basename(path)
    for _, modname, ispkg in pkgutil.iter_modules(path=[path]):
        if exclude and exclude.search(modname):
            continue
        yield "{}.{}".format(name, modname)
        if ispkg:
            for mod2 in iter_all_modules(os.path.join(path, modname), exclude=exclude):
                yield "{}.{}".format(name, mod2)


@pytest.mark.parametrize("modname", iter_all_modules(package.__path__[0]))
def test_import(modname):
    try:
        import_module(modname)
    except Exception as exc:
        if IGNORE.search(str(exc)):
            pytest.skip(str(exc))
        raise


if __name__ == "__main__":
    if "-v" not in " ".join(sys.argv[1:]):  # default to verbose
        sys.argv.append("-v")
    sys.argv.append("-rs")
    sys.exit(pytest.main(args=[__file__] + sys.argv[1:]))



# Test script for comparing GMM integrator to existing mcsampler integrator in
# RIFT. A simple n-dimensional integrand consisting of a highly-correlated
# Gaussian is used.

from __future__ import print_function
import numpy as np
from scipy.stats import multivariate_normal, norm
from scipy.stats import truncnorm
import matplotlib.pyplot as plt

from RIFT.integrators import mcsampler, mcsamplerEnsemble

### test parameters

# width of domain of integration, same for all dimensions
width = 10.0                                                    
llim = -1 * width / 2
rlim = width / 2
ndim = 1                                                        
mu = np.random.uniform(rlim*0.75,rlim)
sigma = 1
cov = sigma**2
# number of iterations for mcsamplerEnsemble
n_iters = 40                                                    
nmax = 40000                                                    


### define integrand as a single gaussian
def f(x1):
    x = np.array(x1,dtype=float)
    return (norm.pdf(x, loc=mu, scale=sigma))

#print(f(np.random.uniform(llim,rlim,size=100)))

### initialize samplers
sampler = mcsampler.MCSampler()
samplerEnsemble = mcsamplerEnsemble.MCSampler()

### add parameters
params = [str(i) for i in range(ndim)]
for p in params:
    sampler.add_parameter(p, np.vectorize(lambda x:1), 
            prior_pdf=np.vectorize(lambda x:1),
            left_limit=llim, right_limit=rlim,
            adaptive_sampling=True)
    samplerEnsemble.add_parameter(p, left_limit=llim, right_limit=rlim,adaptive_sampling=True)

# number of Gaussian components to use in GMM
n_comp = 1

### integrate
integral_1, var_1, eff_samp_1, _ = sampler.integrate(f, *params, 
        no_protect_names=True, nmax=20000, save_intg=True)
print(" --- finished default --")
integral_2, var_2, eff_samp_2, dict_return = samplerEnsemble.integrate(f, *params, 
        min_iter=n_iters, max_iter=n_iters, correlate_all_dims=True, n_comp=n_comp,super_verbose=True,dict_return=True)
print(" --- finished GMM --")
print("mu sigma in ",mu, sigma)
my_integrator = dict_return["integrator"]
gmm_dict = my_integrator.gmm_dict
my_gmm = gmm_dict[(0,)]  # pull out the specific thing we just optimized
scale_from_unnorm = 0.5*(rlim-llim) # scale factor, does not also account for offset 
mu_sampler =my_gmm.means[0][0]*scale_from_unnorm
sigma_sampler =  np.sqrt(my_gmm.covariances[0])[0,0]*scale_from_unnorm
print("mu, sigma for sampler ",mu_sampler,sigma_sampler) # print mean and covariance

# Validate: draw from sampler
npts_here = 500
my_demo_samples = my_gmm.sample(npts_here)[:,0]
my_demo_samples.sort()
plt.scatter(my_demo_samples,np.arange(npts_here)/(1.*npts_here))
plt.plot(my_demo_samples, truncnorm( (llim-mu)/sigma, (rlim-mu)/sigma, loc=mu,scale=sigma).cdf(my_demo_samples),label='truncnorm(crap)')
plt.xlabel("x")
plt.ylabel("CDF of sampling distribution")
plt.savefig("my_cdf_sampler.png")

### Now pull out the integrator's model
### CDFs

### get our posterior samples as a single array
arr_1 = np.empty((len(sampler._rvs["0"]), ndim))
arr_2 = np.empty((len(samplerEnsemble._rvs["0"]), ndim))
for i in range(ndim):
    arr_1[:,i] = sampler._rvs[str(i)].flatten()
    arr_2[:,i] = samplerEnsemble._rvs[str(i)].flatten()

colors = ["black", "red", "blue", "green", "orange"]

plt.figure(figsize=(10, 8))

for i in range(ndim):
    s = sigma
    ### get sorted samples (for the current dimension)
    x_1 = arr_1[:,i][np.argsort(arr_1[:,i])]
    x_2 = arr_2[:,i][np.argsort(arr_2[:,i])]
    ### plot true cdf
    plt.plot(x_1, truncnorm.cdf(x_1, llim, rlim, mu, s), label="True CDF",
            color=colors[i], linewidth=0.5)
    # NOTE: old mcsampler stores L, mcsamplerEnsemble stores lnL
    L = sampler._rvs["integrand"]
    p = sampler._rvs["joint_prior"]
    ps = sampler._rvs["joint_s_prior"]
    ### compute weights of samples
    weights_1 = (L * p / ps)[np.argsort(arr_1[:,i])]
    L = samplerEnsemble._rvs["integrand"]
    p = samplerEnsemble._rvs["joint_prior"]
    ps = samplerEnsemble._rvs["joint_s_prior"]
    ### compute weights of samples
    weights_2 = (L * p / ps)[np.argsort(arr_2[:,i])]
    y_1 = np.cumsum(weights_1)
    y_1 /= y_1[-1] # normalize
    y_2 = np.cumsum(weights_2)
    y_2 /= y_2[-1] # normalize
    ### plot recovered cdf
    plt.plot(x_1, y_1, "--", label="Recovered CDF, mcsampler", color=colors[i],
            linewidth=2)
    plt.plot(x_2, y_2, ":", label="Recovered CDF, mcsamplerEnsemble",
            color=colors[i], linewidth=2)

plt.legend()

fname = "cdf_adapt_test.png"

print("Saving CDF figure as " + fname + "...")

plt.savefig(fname)



#!/usr/bin/env python

# Example Script Provided by:
# https://htcondor.readthedocs.io/en/latest/auto-redirect.html?category=example&tag=bagman-percent-done

import sys
import glob
try:
    import htcondor
except ImportError:
    error("Failed to import htcondor python bindings", 1)

def error(msg: str, code: int):
    """Print error message and exit with specified exit code"""
    print(f"ERROR: {msg}", file=sys.stderr)
    exit(code)


def parse_args():
    """Parse command line arguments"""
    if len(sys.argv) != 3:
        error("Missing argument(s) Job Id and/or Job return code", 1)

    # Parse this nodes Job ID
    try:
        ID = int(sys.argv[1].split(".")[0])
    except ValueError:
        error(f"Failed to convert Job Id ({sys.argv[1]}) to integer", 1)

    # Parse this nodes exit code to preserve node success/failure based on job exit
    try:
        CODE = int(sys.argv[2])
    except ValueError:
        error(f"Failed to convert Job exit code ({sys.argv[2]}) to integer", 1)

    try:
        IT = int(sys.argv[3])
    except ValueError:
        error("No it",1)

    try:
        TARGET = int(sys.argv[4])
    except ValueError:
        error("No target ", 1)

    return (ID, CODE,IT,TARGET)

def get_job_ad(job_id: int, exit_code: int):
    """Query and return the parent DAGMan proper job ad"""
    DAG_ATTRS = ["Iwd"]
    found = False

    schedd = htcondor.Schedd()

    Iwd = None
    # Get workflow Job ID from this a job history ad for this node
    for ad in schedd.history(f"ClusterId=={job_id}", projection=['ProcId', 'ClusterId', 'JobStatus','Iwd'],match=1):
        print(ad)
        if "Iwd" in ad:
            found = True
            Iwd = ad['Iwd']

    if not found:
        error(f"Failed to query job ad for cluster {job_id}", exit_code)


    return Iwd

def main():
    # Threshold to exit if  we reach a threshold of samples
    THRESHOLD = 1000

    job_id, exit_code, it, THRESHOLD = parse_args()

    Iwd = get_job_ad(job_id, exit_code)

    # work size
    n_work = len(glob.glob(Iwd + "/overlap-grid-*.xml.gz"))
    # net n_eff. Using RIFT default n_eff (not n_ESS)
    n_eff_net = 0
    for fname in glob.glob(Iwd+"/overlap-grid*[0-9]+annotation.dat"):
        dat = np.loadtxt(fname) # single line file
        n_eff_net += dat[-1] # last column


    if n_eff_net >= THRESHOLD:
        print(f"CIP jobs have completed {n_eff_net}% effective samples")
        sys.exit(124)

    sys.exit(exit_code)

if __name__ == "__main__":
    main()



#! /usr/bin/env python
#
# util_AddExtrXML.py
# 
# GOAL
#   - to join XML files of ILE extrinsic output  
#   - uses ligolw_add, but combines the XML files in stages by finding a pattern from the filename string
# EXAMPLE
#   - util_AddExtrXML.py iteration_6_ile/EXTR_out-*.xml_*_.xml.gz --output OUTPUT.xml.gz


import sys
from optparse import OptionParser
import numpy as np
try:
    import h5py
except:
    print(" - no h5py - ")

import glob
import os
import subprocess 
import itertools


def chunk(arr_range, arr_size):
    arr_range = iter(arr_range)
    return iter(lambda: list(itertools.islice(arr_range, arr_size)), ())

optp = OptionParser()
optp.add_option("--output",default='merged_output',type=str,help="Merged output file")
opts, args = optp.parse_args()

P_list = []
for pat in args:
    fname_list = glob.glob(pat)
    P_list += fname_list
print("Strings identified")


batches = []
while P_list:
    if len(P_list) > 1000:
      new_batch, P_list = P_list[:1000],P_list[1000:]
      batches += [new_batch]
    else:
      batches += [P_list]
      P_list = []

P_list_list=[' ' for i in range(len(batches))]
for j in np.arange(0,len(batches)):
    for i in batches[j]:
        P_list_list[j] += ' '+ i
print("List of strings created")

for i in range(len(P_list_list)):
    print(i, ' EXTR-INTERIM_'+str(i)+'.xml.gz', batches[i][:2])
    Grid = subprocess.run(['ligolw_add '+ P_list_list[i]+ ' --output  EXTR-INTERIM_'+str(i)+'.xml.gz'], shell=True,capture_output=True)
print("Intermediate grids generated; Joining all now")

subprocess.run(['ligolw_add EXTR-INTERIM_*.xml.gz --output '+ opts.output],shell=True, capture_output=True, text=True)
for name in glob.iglob(os.getcwd()+'/EXTR-INTERIM_*.xml.gz', recursive=True):
    os.remove(name)




import copy
import numpy
import lal
from lalinference.rapid_pe import lalsimutils
m1m2 = numpy.vectorize(lalsimutils.m1m2)

#
# Refinement strategies
#

# Take midpoint between target point and neighbor
def midpoint(pt1, pt2):
    diff = pt2 - pt1
    mid = diff / 2
    return pt1 + mid

class Cell(object):
    def __init__(self, boundaries, center=None):
        self._bounds = boundaries
        if center is not None:
            self._center = center
            assert len(self._center) == len(self._bounds)
            assert all([b[0] < c < b[1] for c, b in zip(self._center, self._bounds)])
        else:
            self._center = [(a+b)/2.0 for a, b in self._bounds]

    def divide(self):
        """
        Subdivide once along each dimension, recursively.
        """
        return self.__recursive_divide(len(self._bounds)-1)
            
    def __recursive_divide(self, dim):
        """
        Do not call directly!
        """

        if dim > 0:
            cells = self.__recursive_divide(dim-1)
        else:
            cells = [self]

        divided_cells = []
        for cell in cells:
            d1, d2 = copy.deepcopy(cell), copy.deepcopy(cell)
            # Divide left
            d1._bounds[dim][1] = self._center[dim]
            d1._center[dim] = (self._bounds[dim][0] + self._center[dim]) / 2.0
            # Divide right
            d2._bounds[dim][0] = self._center[dim]
            d2._center[dim] = (self._bounds[dim][1] + self._center[dim]) / 2.0
            divided_cells.extend([d1, d2])

        return divided_cells

    @staticmethod
    def make_cell_from_boundaries(inpt_pt, pts, symmetric=True):
        """
        Construct a 'virtual cell' from the extent of the points, centered on inpt_pt. If symmetric is True, the boundaries will be symmetric about the input point, if False, the cell will follow the extent of the points on either side.
        """
        cell_bounds = []

        pts = numpy.array(pts)

        # Find the extent of points in each dimension
        ext_right = numpy.max(pts, axis=0)
        ext_left = numpy.min(pts, axis=0)
        for el, er, pt in zip(ext_left, ext_right, inpt_pt):
            if symmetric:
                max_ext = max(abs(pt - el), abs(pt - er))
                bound = (pt - max_ext, pt + max_ext)
            else:
                bound = (el, er)
            cell_bounds.append(bound)

        return Cell(numpy.array(cell_bounds), inpt_pt)

#
# Utilities
#
def prune_duplicate_pts(pts):
    """
    Remove identical points from list.
    """
    return numpy.array(list(set([tuple(pt) for pt in pts])))

#
# Coordinate transformations
#
def transform_m1m2_mceta(m1, m2):
    return lalsimutils.Mceta(m1, m2)

def transform_mceta_m1m2(mc, eta):
    return m1m2(mc, eta)

__prefac_0 = 5. / 256 / numpy.pi
__prefac_3 = 1. / 8
__dim_mass = lal.G_SI / lal.C_SI**3 * lal.MSUN_SI
def transform_m1m2_tau0tau3(m1, m2, flow=40.):
    mt = m1 + m2
    eta = m1 * m2 / mt**2
    mt *= numpy.pi * flow * __dim_mass
    return (__prefac_0 / flow / eta * mt**(-5./3), __prefac_3 / flow / eta * mt**(-2./3))

__prefac_tau = 5. / 32 / numpy.pi
def transform_tau0tau3_m1m2(tau0, tau3, flow=40.):
    mt = __prefac_tau / numpy.pi / flow * tau3 / tau0
    eta = 1.0 / 8 / flow / tau3 * (tau0 / __prefac_tau / tau3)**(2./3)
    m1, m2 = m1m2(mt*eta**(3./5), eta)
    return m1 / __dim_mass, m2 / __dim_mass

VALID_TRANSFORMS_MASS = { \
    "mchirp_eta": transform_m1m2_mceta,
    "tau0_tau3": transform_m1m2_tau0tau3,
    None: None
}

INVERSE_TRANSFORMS_MASS = { \
    transform_m1m2_mceta: transform_mceta_m1m2,
    transform_m1m2_tau0tau3: transform_tau0tau3_m1m2,
    None: None
}

def apply_transform(pts, intr_prms, mass_transform=None):
    # You know what... recarrays are dumb, and so's your face.
    # FIXME: Why does numpy want me to repack this into tuples!?
    #tpts = numpy.array([tuple(pt) for pt in pts.T], dtype = numpy.dtype([(a ,"float64") for a in intr_prms]))
    m1_idx, m2_idx = intr_prms.index("mass1"), intr_prms.index("mass2")
    if mass_transform:
       pts[:,m1_idx], pts[:,m2_idx] = VALID_TRANSFORMS_MASS[mass_transform](pts[:,m1_idx], pts[:,m2_idx])

    # Independent transforms go here

    return pts

def apply_inv_transform(pts, intr_prms, mass_transform=None):
    m1_idx, m2_idx = intr_prms.index("mass1"), intr_prms.index("mass2")
    if mass_transform:
       pts[:,m1_idx], pts[:,m2_idx] = INVERSE_TRANSFORMS_MASS[VALID_TRANSFORMS_MASS[mass_transform]](pts[:,m1_idx], pts[:,m2_idx])

    # Independent transforms go here

    return pts



#! /usr/bin/env python
#
# util_RandomizeOverlapOrder.py
#
# 
# GOAL
#   - take an overlap.xml.gz (SimInspiralTable), and randomize its order.  
#   - Important when merging files from many workers, to avoid accidentally using only the output from one of them.  Run after ligolw_add
#   - takes equal number from each file by default ... WARNING THIS MEANS WE CAN HAVE PATHOLOGICAL SIZE LIMITS


import sys
from optparse import OptionParser
import numpy as np
from igwn_ligolw import utils, table, lsctables, ligolw
try:
    import h5py
except:
    print(" - no h5py - ")

import RIFT.lalsimutils as lalsimutils

# Contenthandlers : argh
#   - http://software.ligo.org/docs/glue/
lsctables.use_in(ligolw.LIGOLWContentHandler)

optp = OptionParser()
optp.add_option("--fref",default=20,type=float,help="Reference frequency. Depending on approximant and age of implementation, may be ignored")
optp.add_option("--n-min",default=20,type=int,help="Minimum size of file to include")
optp.add_option("--output-file",default='merged_output',type=str,help="Merged output file")
optp.add_option("--verbose",action='store_true',help="Print messages")
opts, args = optp.parse_args()


P_list = []
P_list_list =[]
n_list = []
for fname in args:
    P_list_this_file = lalsimutils.xml_to_ChooseWaveformParams_array(fname)
    if len(P_list_this_file)<opts.n_min:
        if opts.verbose: print("  Skipping ", fname)
        continue
    P_list_list.append(P_list_this_file)
    n_list.append(len(P_list_this_file))

if len(n_list) <1:
    raise Exception(" util_RandomizeOverlapOrder : Failure, not enough points in each file greater than {} ".format(opts.n_min))
n_min = np.min(n_list)
for indx in np.arange(len(P_list_list)):
    indx_to_take = np.random.choice(np.arange(len(P_list_list[indx])),size=n_min)
    P_to_add = [P_list_list[indx][a] for a in indx_to_take]
    P_list += P_to_add

lalsimutils.ChooseWaveformParams_array_to_xml(P_list,opts.output_file)



#! /usr/bin/env python

from itertools import combinations
import numpy as np
import fileinput, sys
import argparse

parser = argparse.ArgumentParser()
parser.add_argument("--prefix",default="",help="prefix appended (eg command) before each output line")
opts  = parser.parse_args()


#lines =list(map(lambda x:x.rstrip(),fileinput.input('')))
lines = list(map(lambda x:x.rstrip(), sys.stdin.readlines()))
lines 
#print(" Input ", lines)


res = list(map(list,combinations(lines,2)))
#print(res)
#res = list(map(lambda x:x.sorted(),res))
#print(res)
for indx in np.arange(len(res)):
   print("{} --samples {} --samples {} ".format(opts.prefix,res[indx][0], res[indx][1]))



"""
Source functions to match how RIFT generates waveforms
"""

import numpy as np
import lal
import lalsimulation as lalsim
import RIFT.lalsimutils as lalsimutils
from RIFT.likelihood.factored_likelihood import internal_hlm_generator
import matplotlib
try:
    from bilby.core import utils
except:
    True

has_GWS=False  # make sure defined in top-level scope
try:
        import RIFT.physics.GWSignal as rgws
        has_GWS=True
except:
        has_GWS=False


def RIFT_lal_binary_black_hole_orig(
        frequency_array, mass_1, mass_2, luminosity_distance, spin_1x, spin_1y, spin_1z,
        spin_2x, spin_2y, spin_2z, lambda_1, lambda_2, iota, phase, **kwargs):

    waveform_kwargs = dict(
        waveform_approximant='SEOBNRv4PHM', reference_frequency=15.0,
        minimum_frequency=15.0, maximum_frequency=frequency_array[-1], Lmax=4,
        sampling_frequency=2*frequency_array[-1],
        extra_waveform_kwargs={})
    waveform_kwargs.update(kwargs)
    waveform_approximant = waveform_kwargs['waveform_approximant']
    reference_frequency = waveform_kwargs['reference_frequency']
    minimum_frequency = waveform_kwargs['minimum_frequency']
    maximum_frequency = waveform_kwargs['maximum_frequency']
    sampling_frequency = waveform_kwargs['sampling_frequency']
    Lmax = waveform_kwargs['Lmax']
    waveform_dictionary = waveform_kwargs.get(
        'lal_waveform_dictionary', lal.CreateDict()
    )
    h_method = 'hlmoft'
    extra_waveform_kwargs = waveform_kwargs['extra_waveform_kwargs']
    if 'h_method' in kwargs:
        h_method = kwargs['h_method']

    approximant = lalsim.GetApproximantFromString(waveform_approximant)
    
    P = lalsimutils.ChooseWaveformParams()
    P.m1 = mass_1 * lal.MSUN_SI
    P.m2 = mass_2 * lal.MSUN_SI
    P.s1x = spin_1x; P.s1y = spin_1y; P.s1z = spin_1z
    P.s2x = spin_2x; P.s2y = spin_2y; P.s2z = spin_2z
    P.lambda1 = lambda_1; P.lambda2 = lambda_2
    P.deltaT = 1./(sampling_frequency)

    P.fmin = float(minimum_frequency)
    P.fmax = float(maximum_frequency)
    P.fref = float(reference_frequency)
    P.deltaF=frequency_array[1]-frequency_array[0]
    P.incl = iota
    P.phiref = phase
    P.dist=luminosity_distance*lal.PC_SI*1e6
    P.approx = approximant
    P.taper = lalsim.SIM_INSPIRAL_TAPER_START

    if h_method == 'hlmoft':
        # Waveform generator used internally in RIFT. ILE internally assumes phiref==0, so set this.
        # Note ILE also assumes L-frame waveforms, so this will not work as expected for J-frame output
        # Note several underlying interfaces like ChooseTDModes will enforce these conditions already, but not all. Better safe than sorry.
        P.phiref = 0
        P.incl = 0  # L direction frame
        hlmT = lalsimutils.hlmoft(P,Lmax=Lmax,extra_waveform_kwargs=extra_waveform_kwargs) # extra needed to control ChooseFDWaveform
        P.phiref = phase
        P.incl =  iota # restore
        h22T = hlmT[(2,2)]
        hT = lal.CreateCOMPLEX16TimeSeries("hoft", h22T.epoch, h22T.f0, h22T.deltaT, h22T.sampleUnits, h22T.data.length)
        hT.data.data = np.zeros(hT.data.length)

        # combine modes
        phase_offset = 0#np.pi/2 # TODO this could be a different value i.e. np.pi/2
        for mode in hlmT:
            hT.data.data += hlmT[mode].data.data * lal.SpinWeightedSphericalHarmonic(
                P.incl,phase_offset - 1.0*P.phiref, -2, int(mode[0]), int(mode[1]))
    elif h_method == 'gws_hlmoft':
        # gwsignal specific
        P.phiref = 0
        P.incl = 0  # L direction frame
        hlmT = rgws.hlmoft(P,Lmax=Lmax,approx_string=waveform_approximant,**extra_waveform_kwargs)
        P.phiref = phase
        P.incl =  iota # restore
        h22T = hlmT[(2,2)]
        hT = lal.CreateCOMPLEX16TimeSeries("hoft", h22T.epoch, h22T.f0, h22T.deltaT, h22T.sampleUnits, h22T.data.length)
        hT.data.data = np.zeros(hT.data.length)

        # combine modes
        phase_offset = 0#np.pi/2 # TODO this could be a different value i.e. np.pi/2
        for mode in hlmT:
            hT.data.data += hlmT[mode].data.data * lal.SpinWeightedSphericalHarmonic(
                P.incl,phase_offset - 1.0*P.phiref, -2, int(mode[0]), int(mode[1]))
    else:
        # Backstop waveform generator. Includes all L modes. Use cases where waveforms are J-frame calculations for hlm.
        hT = lalsimutils.complex_hoft(P)
    tvals = lalsimutils.evaluate_tvals(hT)
    t_max = tvals[np.argmax(np.abs(hT.data.data))]

    # end max is cutting the signal such that it ends 2s after merger
    n_max = np.argmax(np.abs(hT.data.data))

    hp = lal.CreateREAL8TimeSeries("h(t)", hT.epoch, hT.f0, hT.deltaT, hT.sampleUnits, hT.data.length)
    hp.data.data = np.real(hT.data.data)
    hc = lal.CreateREAL8TimeSeries("h(t)", hT.epoch, hT.f0, hT.deltaT, hT.sampleUnits, hT.data.length)
    hc.data.data = -np.imag(hT.data.data)

    lalsim.SimInspiralREAL8WaveTaper(hp.data, P.taper)
    lalsim.SimInspiralREAL8WaveTaper(hc.data, P.taper)

    h_plus = hp.data.data
    h_plus = np.concatenate([h_plus[n_max:], h_plus[:n_max]])
    h_cross = hc.data.data
    h_cross = np.concatenate([h_cross[n_max:], h_cross[:n_max]])

    hf_p, freqs = utils.nfft(h_plus, sampling_frequency)
    hf_c, freqs = utils.nfft(h_cross, sampling_frequency)

    #hf_p *= np.exp(2j*np.pi * freqs * tvals[0])
    #hf_c *= np.exp(2j*np.pi * freqs * tvals[0])

    return dict(plus=hf_p, cross=hf_c)


def RIFT_lal_binary_black_hole(
        frequency_array, mass_1, mass_2, luminosity_distance, spin_1x, spin_1y, spin_1z,
        spin_2x, spin_2y, spin_2z,iota, phase, **kwargs):

    waveform_kwargs = dict(
        waveform_approximant='SEOBNRv4PHM', reference_frequency=15.0,
        minimum_frequency=15.0, maximum_frequency=frequency_array[-1], Lmax=4,
        sampling_frequency=2*frequency_array[-1],
        extra_waveform_kwargs={})
    waveform_kwargs.update(kwargs)
    waveform_approximant = waveform_kwargs['waveform_approximant']
    reference_frequency = waveform_kwargs['reference_frequency']
    minimum_frequency = waveform_kwargs['minimum_frequency']
    maximum_frequency = waveform_kwargs['maximum_frequency']
    sampling_frequency = waveform_kwargs['sampling_frequency']
    Lmax = waveform_kwargs['Lmax']
    waveform_dictionary = waveform_kwargs.get(
        'lal_waveform_dictionary', lal.CreateDict()
    )
    h_method = 'hoft'
    extra_waveform_kwargs = waveform_kwargs['extra_waveform_kwargs']
    if 'h_method' in kwargs:
        h_method = kwargs['h_method']

    approximant = lalsim.GetApproximantFromString(waveform_approximant)
    
    P = lalsimutils.ChooseWaveformParams()
    P.m1 = mass_1 * lal.MSUN_SI
    P.m2 = mass_2 * lal.MSUN_SI
    P.s1x = float(spin_1x); P.s1y = float(spin_1y); P.s1z = float(spin_1z)
    P.s2x = float(spin_2x); P.s2y = float(spin_2y); P.s2z = float(spin_2z)
    P.deltaT = 1./(sampling_frequency)

    P.fmin = float(minimum_frequency)
    P.fmax = float(maximum_frequency)
    P.fref = float(reference_frequency)
    P.deltaF=frequency_array[1]-frequency_array[0]
    P.incl = float(iota)
    P.phiref = float(phase)
    P.dist=luminosity_distance*lal.PC_SI*1e6
    P.approx = approximant
    P.taper = lalsim.SIM_INSPIRAL_TAPER_START
    
    if h_method == 'hlmoft':
        # Waveform generator used internally in RIFT. ILE internally assumes phiref==0, so set this.
        # Note ILE also assumes L-frame waveforms, so this will not work as expected for J-frame output
        # Note several underlying interfaces like ChooseTDModes will enforce these conditions already, but not all. Better safe than sorry.
        P.phiref = 0
        P.incl = 0  # L direction frame
        hlmT = lalsimutils.hlmoft(P,Lmax=Lmax,extra_waveform_kwargs=extra_waveform_kwargs) # extra needed to control ChooseFDWaveform
        P.phiref = phase
        P.incl =  iota # restore

        # combine modes
        hT = lalsimutils.hoft_from_hlm(hlmT, P, return_complex=True)
        
    elif h_method == 'gws_hlmoft':
        # gwsignal specific
        P.phiref = 0
        P.incl = 0  # L direction frame
        hlmT = rgws.hlmoft(P,Lmax=Lmax,approx_string=waveform_approximant,**extra_waveform_kwargs)
        P.phiref = phase
        P.incl =  iota # restore

        # combine modes
        hT = lalsimutils.hoft_from_hlm(hlmT, P, return_complex=True)

    elif h_method == 'internal_hlmoft':
        # gwsignal specific
        P.phiref = 0
        P.incl = 0  # L direction frame
        hlmF_1, _= factored_likelihood.internal_hlm_generator(P_copy, opts.Lmax, use_gwsignal=opts.use_gwsignal, use_gwsignal_approx=opts.approximant,ROM_group=opts.rom_group,ROM_param=opts.rom_param, extra_waveform_kwargs=extra_waveform_args, **extra_args)
        hlmT_1  = {}
        for mode in hlmF_1:
            #print(mode,hlmF_1[mode].data.data[0])
            hlmT_1[mode] = lalsimutils.DataInverseFourier(hlmF_1[mode])
        P.phiref = phase
        P.incl =  iota # restore

        # combine modes
        hT = lalsimutils.hoft_from_hlm(hlmT_1, P, return_complex=True)

        
    else:
        # Backstop waveform generator. Includes all L modes. Use cases where waveforms are J-frame calculations for hlm.
        hT = lalsimutils.complex_hoft(P)
    tvals = lalsimutils.evaluate_tvals(hT)
    t_max = tvals[np.argmax(np.abs(hT.data.data))]

    # end max is cutting the signal such that it ends 2s after merger
    n_max = np.argmax(np.abs(hT.data.data))

    hp = lal.CreateREAL8TimeSeries("h(t)", hT.epoch, hT.f0, hT.deltaT, hT.sampleUnits, hT.data.length)
    hp.data.data = np.real(hT.data.data)
    hc = lal.CreateREAL8TimeSeries("h(t)", hT.epoch, hT.f0, hT.deltaT, hT.sampleUnits, hT.data.length)
    hc.data.data = -np.imag(hT.data.data)

    lalsim.SimInspiralREAL8WaveTaper(hp.data, P.taper)
    lalsim.SimInspiralREAL8WaveTaper(hc.data, P.taper)

    h_plus = hp.data.data
    h_plus = np.concatenate([h_plus[n_max:], h_plus[:n_max]])
    h_cross = hc.data.data
    h_cross = np.concatenate([h_cross[n_max:], h_cross[:n_max]])

    hf_p, freqs = utils.nfft(h_plus, sampling_frequency)
    hf_c, freqs = utils.nfft(h_cross, sampling_frequency)

    #hf_p *= np.exp(2j*np.pi * freqs * tvals[0])
    #hf_c *= np.exp(2j*np.pi * freqs * tvals[0])

    # PROBLEM: now we need to interpolate to frequency_array : our grid does not correspond to frequency_array in general
    if frequency_array[0] >0:
        hf_p_out = np.zeros(frequency_array.shape,dtype=np.complex128)
        hf_c_out = np.zeros(frequency_array.shape,dtype=np.complex128)
        # interpolate in real, imag
        hf_p_out += np.interp(frequency_array, freqs, np.real(hf_p))
        hf_p_out += 1j* np.interp(frequency_array, freqs, np.imag(hf_p))
        hf_c_out += np.interp(frequency_array, freqs, np.real(hf_c))
        hf_c_out += 1j* np.interp(frequency_array, freqs, np.imag(hf_c))
        hf_p = hf_p_out
        hf_c = hf_c_out
    
    return dict(plus=hf_p, cross=hf_c)


if __name__ == '__main__':
    from matplotlib import pyplot as plt

    # For testing purposes we evaluate one waveform and plot it here
    # test with two different maximum frequencies
    frequency_array = np.arange(15, 1024., 1./8)
    waveform_kwargs = {'Lmax':4, 'maximum_frequency':1024, 'minimum_frequency':10}
    waveform_polarizations = RIFT_lal_binary_black_hole(
        frequency_array, 60., 55., 400., 0.0, 0.0, 0.1,
        0.0, 0.0, 0.1, iota=np.pi/4, phase=np.pi/2, **waveform_kwargs)

    hf_p = waveform_polarizations['plus']
    hf_c = waveform_polarizations['cross']

    waveform_kwargs = {'Lmax':4, 'maximum_frequency':4096, 'minimum_frequency':10}
    waveform_polarizations = RIFT_lal_binary_black_hole(
        frequency_array, 60., 55., 400., 0.0, 0.0, 0.1,
        0.0, 0.0, 0.1, iota=np.pi/4, phase=np.pi/2, **waveform_kwargs)

    hf_p2 = waveform_polarizations['plus'][:int(len(waveform_polarizations['plus'])/4)]
    hf_c2 = waveform_polarizations['cross'][:int(len(waveform_polarizations['plus'])/4)]

    plt.axvline(15, color='k')

    plt.loglog(np.linspace(0,frequency_array[-1],len(hf_p)), np.abs(hf_p), color='C0')
    plt.loglog(np.linspace(0,frequency_array[-1],len(hf_p)), np.abs(hf_c), color='C0', linestyle='--')
    plt.loglog(np.linspace(0,frequency_array[-1],len(hf_p2)), np.abs(hf_p2), color='C1')
    plt.loglog(np.linspace(0,frequency_array[-1],len(hf_p2)), np.abs(hf_c2), color='C1', linestyle='--')
    plt.show()
    plt.clf()

    # be aware shifts by 2pi are common, plot so we can see that shift
    plt.semilogx(np.linspace(0,frequency_array[-1],len(hf_p)), np.unwrap(np.angle(hf_p)), color='C0')
    plt.semilogx(np.linspace(0,frequency_array[-1],len(hf_p)), np.unwrap(np.angle(hf_c)), color='C0', linestyle='--')
    plt.semilogx(np.linspace(0,frequency_array[-1],len(hf_p2)), np.unwrap(np.angle(hf_p2)), color='C1')
    plt.semilogx(np.linspace(0,frequency_array[-1],len(hf_p2)), np.unwrap(np.angle(hf_c2)), color='C1', linestyle='--')
    plt.show()
    plt.clf()



#!/usr/bin/env python
import sys
import os
import functools
import itertools
import bisect

import healpy

import matplotlib
matplotlib.use("Agg")
from matplotlib import pyplot
from mpl_toolkits.basemap import Basemap

import numpy
import scipy.integrate
import scipy.special

from lalinference.bayestar import fits as bfits
from lalinference.bayestar import plot as bplot

import RIFT.integrators.mcsampler as mcsampler
from statutils import cumvar


__author__ = "Chris Pankow <pankow@gravity.phys.uwm.edu>"

#
# set up bounds on parameters
#

# Polarization angle
#psi_min, psi_max = 0, 2*numpy.pi
psi_min, psi_max = -2*numpy.pi, 2*numpy.pi
psi_val, psi_width = numpy.pi/2, numpy.pi/6
# RA and dec
ra_min, ra_max = 0, 2*numpy.pi
#ra_min, ra_max = -2*numpy.pi, 2*numpy.pi
ra_val, ra_width = numpy.pi/4, 5*numpy.pi/180
dec_min, dec_max = -numpy.pi/2, numpy.pi/2
dec_val, dec_width = -numpy.pi/4, 5*numpy.pi/180
# Inclination angle
inc_min, inc_max = -numpy.pi/2, numpy.pi/2
inc_val, inc_width = -numpy.pi/3, 10*numpy.pi/180
# orbital phi
phi_min, phi_max = 0, 2*numpy.pi
phi_val, phi_width = numpy.pi/5, 10*numpy.pi/180
# distance
dist_min, dist_max = 0.0, 100.0
dist_val, dist_width = 25.0, 25.0


samp = mcsampler.MCSampler()

#
# Test 6: sampling, pin parameters
#
# Uniform sampling, auto-cdf inverse
samp.add_parameter("psi", functools.partial(mcsampler.gauss_samp, psi_val, 2*psi_width), None, psi_min, psi_max, prior_pdf=lambda x: 1)
samp.add_parameter("ra", functools.partial(mcsampler.uniform_samp_vector, ra_min, ra_max), None, ra_min, ra_max, prior_pdf=lambda x: 1)
samp.add_parameter("dec", functools.partial(mcsampler.uniform_samp_vector, dec_min, dec_max), None, dec_min, dec_max, prior_pdf=lambda x: 1)
samp.add_parameter("phi", functools.partial(mcsampler.uniform_samp_vector, phi_min, phi_max), None, phi_min, phi_max, prior_pdf=lambda x: 1)
samp.add_parameter("inc", functools.partial(mcsampler.gauss_samp, inc_val, 2*inc_width), None, inc_min, inc_max, prior_pdf=lambda x: 1)
samp.add_parameter("dist", functools.partial(mcsampler.uniform_samp_vector, dist_min, dist_max), None, dist_min, dist_max, prior_pdf=lambda x: 1)

#
# Full 6-D test
#

a, b, c, d, e, f = 2*psi_width**2, 2*ra_width**2, 2*dec_width**2, 2*inc_width**2, 2*phi_width**2, 2*dist_width**2
norm = 1.0/numpy.sqrt((numpy.pi)**len(samp.params)*a*b*c*d*e*f)
numpy.seterr(invalid="raise")
def integrand(p, r, dec, ph, i, di):
	exponent = -(p-psi_val)**2/a-(r-ra_val)**2/b-(dec-dec_val)**2/c-(i-inc_val)**2/d-(ph-phi_val)**2/e-(di-dist_val)**2/f
	return norm * numpy.exp(exponent)

res, var = samp.integrate(integrand, "psi", "phi", "inc", "dist", neff=100, nmax=int(1e6), ra=ra_val, dec=dec_val)
print("Integral value: %f, stddev %f" % (res, numpy.sqrt(var)))



import sys
import lal
import lalsimulation as lalsim
import lalsimutils as lsu
import effectiveFisher as eff
import dag_utils
import numpy as np
import matplotlib.pyplot as plt
from time import clock
from functools import partial
from six.moves import range

import itertools
from glue.ligolw import utils, ligolw, lsctables, table
from glue.ligolw.utils import process

start = clock()
elapsed_time = lambda: clock()-start

pts_per_job = 10 # How many intrinsic points to pass to each condor job

def write_sngl_params(grid, proc_id):
    sngl_insp_table = lsctables.New(lsctables.SnglInspiralTable, ["mass1", "mass2", "event_id", "process_id"])
    sngl_insp_table.sync_next_id()
    for (m1, m2) in itertools.chain(*grid):
        sngl_insp = sngl_insp_table.RowType()
        sngl_insp.event_id = sngl_insp_table.get_next_id()
        #sngl_insp.mass1, sngl_insp.mass2 = m1/lal.LAL_MSUN_SI, m2/lal.LAL_MSUN_SI
        sngl_insp.mass1, sngl_insp.mass2 = m1, m2
        sngl_insp.process_id = proc_id
        sngl_insp_table.append(sngl_insp)

    return sngl_insp_table

# Setup signal and IP class
m1=1.5*lal.LAL_MSUN_SI
m2=1.35*lal.LAL_MSUN_SI
PSIG = lsu.ChooseWaveformParams(m1=m1, m2=m2, approx=lalsim.TaylorT1)
PTEST = PSIG.copy() # find deltaF for lower end of range we're looking in
PTEST.m1 *= 0.9
PTEST.m2 *= 0.9
PSIG.deltaF = lsu.findDeltaF(PTEST)
IP = lsu.Overlap(fLow=40., deltaF=PSIG.deltaF, psd=lal.LIGOIPsd)
PTMPLT = PSIG.copy()
hfSIG = lsu.norm_hoff(PSIG, IP)
McSIG = lsu.mchirp(m1, m2)
etaSIG = lsu.symRatio(m1, m2)
NMcs = 11
NEtas = 11
param_names = ['Mc', 'eta']

# Find appropriate parameter ranges
param_ranges = eff.find_effective_Fisher_region(PSIG, IP, 0.90, param_names,
        [[0.9*McSIG,1.1*McSIG],[0.05,0.25]])
print("Computing amibiguity function in the range:")
for i, param in enumerate(param_names):
    if param=='Mc' or param=='m1' or param=='m2': # rescale output by MSUN
        print("\t", param, ":", np.array(param_ranges[i])/lal.LAL_MSUN_SI,\
                "(Msun)")
    else:
        print("\t", param, ":", param_ranges[i])

elapsed = elapsed_time()
print("Range-finding took:", elapsed, "(s) for", len(param_names),"parameters\n")

# setup uniform parameter grid for effective Fisher
pts_per_dim = [NMcs, NEtas]
Mcpts, etapts = eff.make_regular_1d_grids(param_ranges, pts_per_dim)
etapts = map(lsu.sanitize_eta, etapts)
McMESH, etaMESH = eff.multi_dim_meshgrid(Mcpts, etapts)
McFLAT, etaFLAT = eff.multi_dim_flatgrid(Mcpts, etapts)
dMcMESH = McMESH - McSIG
detaMESH = etaMESH - etaSIG
dMcFLAT = McFLAT - McSIG
detaFLAT = etaFLAT - etaSIG
grid = eff.multi_dim_grid(Mcpts, etapts)

# Change units on Mc
dMcFLAT_MSUN = dMcFLAT / lal.LAL_MSUN_SI
dMcMESH_MSUN = dMcMESH / lal.LAL_MSUN_SI
McMESH_MSUN = McMESH / lal.LAL_MSUN_SI
McSIG_MSUN = McSIG / lal.LAL_MSUN_SI

# Evaluate ambiguity function on the grid
rhos = np.array(eff.evaluate_ip_on_grid(hfSIG, PTMPLT, IP, param_names, grid))
rhogrid = rhos.reshape(NMcs, NEtas)

# Fit to determine effective Fisher matrix
cut = rhos > 0.99
fitgamma = eff.effectiveFisher(eff.residuals2d, rhos[cut], dMcFLAT_MSUN[cut],
        detaFLAT[cut])
# Find the eigenvalues/vectors of the effective Fisher matrix
gam = eff.array_to_symmetric_matrix(fitgamma)
evals, evecs, rot = eff.eigensystem(gam)

elapsed = elapsed_time() - elapsed
print("Time to compute effective Fisher matrix and its eigensystem:", elapsed)
print("For a grid of size:", pts_per_dim, "\n")

# Print information about the effective Fisher matrix
# and its eigensystem
print("Least squares fit finds g_Mc,Mc = ", fitgamma[0])
print("                        g_Mc,eta = ", fitgamma[1])
print("                        g_eta,eta = ", fitgamma[2])

print("\nFisher matrix:")
print("eigenvalues:", evals)
print("eigenvectors:")
print(evecs)
print("rotation taking eigenvectors into Mc, eta basis:")
print(rot)

#
# Distribute points inside predicted ellipsoid of certain level of overlap
#
match_cntr = 0.90
Nrandpts=200
r1 = np.sqrt(2.*(1.-match_cntr)/np.real(evals[0])) # ellipse radii along eigendirections
r2 = np.sqrt(2.*(1.-match_cntr)/np.real(evals[1]))
# Get pts. inside an ellipsoid oriented along eigenvectors...
#cart_grid, sph_grid = eff.uniform_random_ellipsoid(Nrandpts, r1, r2)
Nrad = 10
Nspokes = 40
ph0 = np.arctan(np.abs(r1) * (rot[0,1])/(np.abs(r2) * rot[0,0]) )
#ph0 = 0.
print("angle is:", ph0, r1, r2)
#cart_grid, sph_grid = eff.uniform_spoked_ellipsoid(Nrad,Nspokes, [ph0], r1, r2)
cart_grid, sph_grid = eff.linear_spoked_ellipsoid(Nrad, Nspokes, [ph0], r1, r2)
gridT = np.transpose(cart_grid)
xs = gridT[0]
ys = gridT[1]
# Rotate to get coordinates in parameter basis
cart_grid = np.array([ np.real( np.dot(rot, cart_grid[i]))
    for i in range(len(cart_grid)) ])
gridT = np.transpose(cart_grid)
Xs = gridT[0]
Ys = gridT[1]
# Put in convenient units,
# change from parameter differential (i.e. dtheta)
# to absolute parameter value (i.e. theta = theta_true + dtheta)
rand_dMcs_MSUN, rand_detas = tuple(np.transpose(cart_grid)) # dMc, deta
rand_Mcs = rand_dMcs_MSUN * lal.LAL_MSUN_SI + McSIG # Mc (kg)
rand_etas = rand_detas + etaSIG # eta

# Prune points with unphysical values of eta from cart_grid
rand_etas = np.array(map(partial(lsu.sanitize_eta, exception=np.NAN), rand_etas))
cart_grid = np.transpose((rand_Mcs,rand_etas))
phys_cut = ~np.isnan(cart_grid).any(1) # cut to remove unphysical pts
unphys_cut = np.isnan(cart_grid).any(1) # unphysical pts only
cart_grid = cart_grid[phys_cut]
print("Requested", Nrandpts, "points inside the ellipsoid of",\
        match_cntr, "match.")
print("Kept", len(cart_grid), "points with physically allowed parameters.")

# Save grid of mass points to file
#np.savetxt("Mc_eta_pts.txt", cart_grid)
cart_grid2 = np.array([lsu.m1m2(cart_grid[i][0], cart_grid[i][1]) # convert to m1, m2
        for i in range(len(cart_grid))])
cart_grid2 /= lal.LAL_MSUN_SI
#np.savetxt("m1_m2_pts.txt", cart_grid2)
Njobs = int(np.ceil(len(cart_grid2)/float(pts_per_job)))
cart_grid3 = np.array_split(cart_grid2, Njobs)
for i in range(Njobs):
        fname = "m1_m2_pts_%i.txt" % i
        np.savetxt(fname, cart_grid3[i])

elapsed = elapsed_time() - elapsed
print("Time to distribute points, split and write to file:", elapsed)

#dag_utils.write_integrate_likelihood_extrinsic_sub('test')
#dag_utils.write_extrinsic_marginalization_dag(cart_grid2, 'test.sub')

xmldoc = ligolw.Document()
xmldoc.childNodes.append(ligolw.LIGO_LW())
#proc_id = process.register_to_xmldoc(xmldoc, sys.argv[0], opts.__dict__)
proc_id = process.register_to_xmldoc(xmldoc, sys.argv[0], {})
proc_id = proc_id.process_id
xmldoc.childNodes[0].appendChild(write_sngl_params(cart_grid3, proc_id))
utils.write_filename(xmldoc, "m1m2_grid.xml.gz", gz=True)

#
# N.B. Below here, the real code will divy up cart_grid into blocks of intrinsic
# parameters and compute the marginalized likelihood on each point
# in intrinsic parameter space
#
# For testing purposes, simply evaluate the overlap on cart_grid and plot
# to confirm it is placing points in the expected ellipsoid with the
# proper distribution, and the overlap behaves properly.
#

# Evaluate IP on the grid inside the ellipsoid
rhos2 = eff.evaluate_ip_on_grid(hfSIG, PTMPLT, IP, param_names, cart_grid)

# Plot the ambiguity function, effective Fisher and ellipsoid points
plt.figure(1)
plt.title('Ambiguity function')
cntrs = [.97,0.975,0.98,0.985,0.99,0.995,1.0]
plt.contourf(McMESH_MSUN, etaMESH, rhogrid, cntrs, cmap=plt.cm.jet)
plt.colorbar()
plt.scatter(McSIG_MSUN,etaSIG,marker='x',c='k',s=40)
plt.xlabel("Mc")
plt.ylabel("eta")

plt.figure(2)
plt.title('Effective Fisher contours')
plt.contourf(dMcMESH_MSUN, detaMESH,
        eff.evalfit2d(dMcMESH_MSUN, detaMESH, fitgamma),
        cntrs, cmap=plt.cm.jet)
plt.colorbar()
plt.scatter(0.,0.,marker='x',c='k',s=40)
plt.xlabel("$\Delta M_c$")
plt.ylabel("$\Delta \eta$")

plt.figure(3)
plt.title('Random sampling of 0.97 ellipsoid')
plt.scatter(rand_dMcs_MSUN[phys_cut], rand_detas[phys_cut], c=rhos2, cmap=plt.cm.jet)
plt.colorbar()
plt.xlabel("$\Delta M_c$")
plt.ylabel("$\Delta \eta$")

plt.figure(4)
plt.title('Unrotated ellipse')
plt.scatter(xs, ys)
plt.xlabel("$e_1$")
plt.ylabel("$e_2$")
plt.figure(5)
plt.title('Rotated ellipse')
plt.scatter(Xs, Ys)
plt.xlabel("$e_1$")
plt.ylabel("$e_2$")

plt.figure(6)
plt.title('Intrinsic parameter placement for $(1.5+1.35) M_\odot$')
plt.scatter(McSIG_MSUN + rand_dMcs_MSUN[unphys_cut], etaSIG + rand_detas[unphys_cut], c='b', marker='x', label="unphysical points")
plt.scatter(McSIG_MSUN + rand_dMcs_MSUN[phys_cut], etaSIG + rand_detas[phys_cut], c='r', label="physical points")
plt.axhline(0.25, c='k')
plt.xlabel("$M_c (M_\odot)$")
plt.ylabel("$\eta$")
plt.legend(loc='upper left')
plt.show()



#! /usr/bin/env python
#
#


import argparse
import numpy as np
from RIFT.misc.samples_utils import add_field

import sys

import lal
import lalsimulation as lalsim
import RIFT.lalsimutils as lalsimutils


remap_ILE_2_LI = {
 "s1z":"a1z", "s2z":"a2z", 
 "s1x":"a1x", "s1y":"a1y",
 "s2x":"a2x", "s2y":"a2y",
 "chi1_perp":"chi1_perp",
 "chi2_perp":"chi2_perp",
 "chi1":'a1',
 "chi2":'a2',
 "cos_phiJL": 'cos_phiJL',
 "sin_phiJL": 'sin_phiJL',
 "cos_theta1":'costilt1',
 "cos_theta2":'costilt2',
 "theta1":"tilt1",
 "theta2":"tilt2",
  "xi":"chi_eff", 
  "chiMinus":"chi_minus", 
  "delta":"delta", 
  "delta_mc":"delta", 
 "mtot":'mtotal', "mc":"mc", "eta":"eta","m1":"m1","m2":"m2",
  "cos_beta":"cosbeta",
  "beta":"beta",
  "LambdaTilde":"lambdat",
  "DeltaLambdaTilde": "dlambdat",
  "thetaJN":"theta_jn",
  "dist":"distance",'phiref':'phiorb'}
remap_LI_to_ILE = { "a1z":"s1z", "a2z":"s2z", "chi_eff":"xi", "lambdat":"LambdaTilde", 'mtotal':'mtot','distance':'dist', 'ra':'phi', 'dec':'theta','phiorb':'phiref','a1x':'s1x','a1y':'s1y', 'a2x':'s2x', 'a2y':'s2y'}




# Parse arguments
parser = argparse.ArgumentParser()
parser.add_argument("--posterior-file",action='append',help="filename of *.dat file [standard LI output]")
parser.add_argument("--flag-tides-in-composite",action='store_true')
parser.add_argument("--composite-file",type=str,help="filename of all.net file, needed to get peak lnL (i.e., log how much this event is pure noise")
parser.add_argument("--truth-file",type=str, help="file containing the true parameters")
parser.add_argument("--truth-event",type=int, default=0,help="file containing the true parameters")
parser.add_argument("--chi-max",type=int, default=1,help="re-impose chi-max")
parser.add_argument("--parameter", action='append',help="parameter name (ILE). Note source-frame masses are only natively supported for LI")
opts=  parser.parse_args()

# Load in truth file
P_ref = None
if opts.truth_file:
    print(" Loading true parameters from  ", opts.truth_file)
    truth_P_list  =lalsimutils.xml_to_ChooseWaveformParams_array(opts.truth_file)
    P_ref = truth_P_list[opts.truth_event]
else:
    sys.exit(0)

lnL_max = -np.inf
lnL_col = 9
if opts.flag_tides_in_composite:
    lnL_col = 11
if opts.composite_file:
    dat = np.loadtxt(opts.composite_file)
    lnL_max = np.max(dat[:,lnL_col])

# Load in posterior samples
fname = opts.posterior_file[0] # just do the one
samples = np.genfromtxt(fname,names=True)
if 'time' in samples.dtype.names:
    samples = add_field(samples, [('tref',float)]); samples['tref'] = samples['time']
if not 'mtotal' in samples.dtype.names and 'mc' in samples.dtype.names:  # raw LI samples use 
    q_here = samples['q']
    eta_here = q_here/(1+q_here)
    mc_here = samples['mc']
    mtot_here = mc_here / np.power(eta_here, 3./5.)
    m1_here = mtot_here/(1+q_here)
    samples = add_field(samples, [('mtotal', float)]); samples['mtotal'] = mtot_here
    samples = add_field(samples, [('eta', float)]); samples['eta'] = eta_here
    samples = add_field(samples, [('m1', float)]); samples['m1'] = m1_here
    samples = add_field(samples, [('m2', float)]); samples['m2'] = mtot_here * q_here/(1+q_here)

if (not 'theta1' in samples.dtype.names)  and ('a1x' in samples.dtype.names):  # probably does not have polar coordinates
    chiperp_here = np.sqrt( samples['a1x']**2+ samples['a1y']**2)
    if any(chiperp_here > 0):
     chi1_here = np.sqrt( samples['a1z']**2 + chiperp_here**2)
     theta1_here = np.arctan(chiperp_here/ samples['a1z'])
     phi1_here = np.angle(samples['a1x']+1j*samples['a1y'])
     samples = add_field(samples, [('chi1', float)]); samples['chi1'] = chi1_here
     samples = add_field(samples, [('theta1', float)]); samples['theta1'] = theta1_here
     samples = add_field(samples, [('phi1', float)]); samples['phi1'] = phi1_here

    chiperp_here = np.sqrt( samples['a2x']**2+ samples['a2y']**2)
    if any(chiperp_here > 0):
     
     chi2_here = np.sqrt( samples['a2z']**2 + chiperp_here**2)
     theta2_here = np.arctan(chiperp_here/ samples['a2z'])
     phi2_here = np.angle(samples['a2x']+1j*samples['a2y'])
     samples = add_field(samples, [('chi2', float)]); samples['chi2'] = chi1_here
     samples = add_field(samples, [('theta2', float)]); samples['theta2'] = theta1_here
     samples = add_field(samples, [('phi2', float)]); samples['phi2'] = phi1_here

elif "theta1" in samples.dtype.names:
    a1x_dat = samples["a1"]*np.sin(samples["theta1"])*np.cos(samples["phi1"])
    a1y_dat = samples["a1"]*np.sin(samples["theta1"])*np.sin(samples["phi1"])
    chi1_perp = samples["a1"]*np.sin(samples["theta1"])

    a2x_dat = samples["a2"]*np.sin(samples["theta2"])*np.cos(samples["phi2"])
    a2y_dat = samples["a2"]*np.sin(samples["theta2"])*np.sin(samples["phi2"])
    chi2_perp = samples["a2"]*np.sin(samples["theta2"])


    samples = add_field(samples, [('a1x', float)]);  samples['a1x'] = a1x_dat
    samples = add_field(samples, [('a1y', float)]); samples['a1y'] = a1y_dat
    samples = add_field(samples, [('a2x', float)]);  samples['a2x'] = a2x_dat
    samples = add_field(samples, [('a2y', float)]);  samples['a2y'] = a2y_dat
    samples = add_field(samples, [('chi1_perp',float)]); samples['chi1_perp'] = chi1_perp
    samples = add_field(samples, [('chi2_perp',float)]); samples['chi2_perp'] = chi2_perp
    if not 'chi_eff' in samples.dtype.names:
        samples = add_field(samples, [('chi_eff',float)]); samples['chi_eff'] = (samples["m1"]*samples["a1z"]+samples["m2"]*samples["a2z"])/(samples["m1"]+samples["m2"])

# add other derived parameters
if 'a1x' in samples.dtype.names:
    if not ('chi1_perp' in samples.dtype.names):
        chi1_perp = np.sqrt(samples['a1x']**2 + samples['a1y']**2)
        chi2_perp = np.sqrt(samples['a2x']**2 + samples['a2y']**2)
        samples = add_field(samples, [('chi1_perp',float)]); samples['chi1_perp'] = chi1_perp
        samples = add_field(samples, [('chi2_perp',float)]); samples['chi2_perp'] = chi2_perp
    if not 'chi_eff' in samples.dtype.names:
        samples = add_field(samples, [('chi_eff',float)]); samples['chi_eff'] = (samples["m1"]*samples["a1z"]+samples["m2"]*samples["a2z"])/(samples["m1"]+samples["m2"])
        samples = add_field(samples, [('xi',float)]); samples['chi_eff'] = (samples["m1"]*samples["a1z"]+samples["m2"]*samples["a2z"])/(samples["m1"]+samples["m2"])



if 'lambda1' in samples.dtype.names and not ('lambdat' in samples.dtype.names):
    Lt,dLt = lalsimutils.tidal_lambda_tilde(samples['m1'], samples['m2'],  samples['lambda1'], samples['lambda2'])
    samples = add_field(samples, [('lambdat', float)]); samples['lambdat'] = Lt
    samples = add_field(samples, [('dlambdat', float)]); samples['dlambdat'] = dLt


if 'chi1_perp' in samples.dtype.names:
    # impose Kerr limit, if neede
    npts = len(samples["m1"])
    indx_ok =np.arange(npts)
    chi1_squared = samples['chi1_perp']**2 + samples["a1z"]**2
    chi2_squared = samples["chi2_perp"]**2 + samples["a2z"]**2
    indx_ok = np.logical_and(chi1_squared < opts.chi_max ,chi2_squared < opts.chi_max)
    npts_out = np.sum(indx_ok)
    new_samples = np.recarray( (npts_out,), dtype=samples.dtype)
    for name in samples.dtype.names:
        new_samples[name] = samples[name][indx_ok]
    samples = new_samples

# Loop over labelled fields
for param in opts.parameter:
    param_here = param
    if param_here in list(remap_LI_to_ILE.keys()):
        param_here = remap_LI_to_ILE[param_here]
    val_here = P_ref.extract_param(param_here)
    if param in [ 'mc', 'm1', 'm2', 'mtotal']:
        val_here= val_here/lal.MSUN_SI
    if param in ['dist','distance']:
        val_here = val_here/(1e6*lal.PC_SI)
 
    if param in ['phiref','phiorb', 'psi']:   # BUG in injection file, this is workaround
        samples[param] = np.mod(samples[param],np.pi)

    n_ok = np.sum(samples[param] < val_here)
    print(n_ok/(1.*len(samples[param])), end=' ')

if opts.composite_file:
    print(lnL_max)



#! /usr/bin/env python
#
# GOAL
#   - read in parameter XML
#   - assess parameter covariance (in some named parameters)
#   - generate random numbers, based on that covariance, to add to those parameters, 'puffing up' the distribution
#
# USAGE
#   - tries to follow util_CIP_GC
#
# EXAMPLES
#    util_ManualOverlapGrid.py  --skip-overlap --parameter mc --parameter-range [1,2] --parameter eta --parameter-range [0.1,0.2] --parameter s1z --parameter-range [-1,1] --parameter s2z --parameter-range [-1,1] 
#    python util_ParameterPuffball.py  --parameter mc --parameter eta --no-correlation "['mc','eta']" --parameter s1z --parameter s2z --inj-file ./overlap-grid.xml.gz  --no-correlation "['mc','s1z']"
#   python util_ParameterPuffball.py  --parameter mc --parameter eta --parameter s1z --parameter s2z --inj-file ./overlap-grid.xml.gz   --force-away 0.4

# PROBLEMS
#    - if points are too dense (i.e, if the output size gets too large) then we will reject everything, even for uniform placement.  
#    - current implementation produces pairwise distance matrix, so can be memory-hungry for many points


import argparse
import sys
import numpy as np
import numpy.lib.recfunctions
import functools
import itertools




parser = argparse.ArgumentParser()
parser.add_argument("--inj-file", help="Name of XML file")
parser.add_argument("--inj-file-out", default="output-puffball", help="Name of XML file")
parser.add_argument("--puff-factor", default=1,type=float)
parser.add_argument("--force-away", default=0,type=float,help="If >0, uses the icov to compute a metric, and discards points which are close to existing points")
parser.add_argument("--approx-output",default="SEOBNRv2", help="approximant to use when writing output XML files.")
parser.add_argument("--fref",default=None,type=float, help="Reference frequency used for spins in the ILE output.  (Since I usually use SEOBNRv3, the best choice is 20Hz). Default is to use what is in the original overlap-grid.xml.gz file")
parser.add_argument("--fmin",type=float,default=None,help="Min frequency, default is to use what is in original file")
parser.add_argument("--parameter", action='append', help="Parameters used as fitting parameters AND varied at a low level to make a posterior")
parser.add_argument("--no-correlation", type=str,action='append', help="Pairs of parameters, in format [mc,eta]  The corresponding term in the covariance matrix is eliminated")
#parser.add_argument("--parameter-implied", action='append', help="Parameter used in fit, but not independently varied for Monte Carlo")
parser.add_argument("--random-parameter", action='append',help="These parameters are specified at random over the entire range, uncorrelated with the grid used for other parameters.  Use for variables which correlate weakly with others; helps with random exploration")
parser.add_argument("--random-parameter-range", action='append', type=str,help="Add a range (pass as a string evaluating to a python 2-element list): --parameter-range '[0.,1000.]'   MUST specify ALL parameter ranges (min and max) in order if used.  ")
parser.add_argument("--mc-range",default=None,help="Chirp mass range [mc1,mc2]. Important if we have a low-mass object, to avoid wasting time sampling elsewhere.")
parser.add_argument("--eta-range",default=None,help="Eta range. Important if we have a BNS or other item that has a strong constraint.")
parser.add_argument("--mtot-range",default=None,help="Chirp mass range [mc1,mc2]. Important if we have a low-mass object, to avoid wasting time sampling elsewhere.")
parser.add_argument("--downselect-parameter",action='append', help='Name of parameter to be used to eliminate grid points ')
parser.add_argument("--downselect-parameter-range",action='append',type=str)
parser.add_argument("--enforce-duration-bound",default=None,type=float,help="If present, enforce a duration bound. Used to prevent grid placement for obscenely long signals, when the window size is prescribed")
parser.add_argument("--regularize",action='store_true',help="Add some ad-hoc terms based on priors, to help with nearly-singular matricies")
opts=  parser.parse_args()

print(" Arguments: ", ' '.join(sys.argv[1:]))
print(" Inputs: ", opts.inj_file)
print(" Outputs: ", opts.inj_file_out)

# Export
import os
os.system(" touch "+opts.inj_file_out + ".xml.gz")

    





import numpy as np
from matplotlib import pylab as plt

import RIFT.integrators.mcsampler as mcsampler
#import ourio

#import dill # so I can pickle lambda functions: https://stackoverflow.com/questions/25348532/can-python-pickle-lambda-functions?utm_medium=organic&utm_source=google_rich_qa&utm_campaign=google_rich_qa


# Integrate a constant function
samplerPrior = mcsampler.MCSampler()
samplerPrior.add_parameter('x', pdf=np.vectorize(lambda x: 1.3), cdf_inv=None, left_limit=-1.5,right_limit=1)  # is this correctly renormalized?
#    Note code above RELIES on our ability to set a default prior (of unity!) for x!
#    AND our ability to construct a cdf_inverse function, which is done approximately, for sampling
# Do an MC integral with this sampler (=the measure specified by the sampler).
def my_func(x):
    return np.ones(len(x))
ret,var, neff,dict_return = samplerPrior.integrate(my_func, 'x', nmax=1e5,verbose=True)
print("Integral of 1 over this range ", [samplerPrior.llim['x'] ,samplerPrior.rlim['x'] ], " is ", ret, " needs to be ", samplerPrior.rlim['x'] -  samplerPrior.llim['x'] ," and (small)")


# Do a gaussian integral.  Create normalizatio interactively
samplerNewPrior = mcsampler.MCSampler()
samplerNewPrior.add_parameter('y', pdf=(lambda x: np.exp(-x**2/2.)), cdf_inv=None, left_limit=-1,right_limit=1, prior_pdf = np.vectorize(lambda x: 1/2.)) # normalized prior!
ret,var, neff,dict_return = samplerNewPrior.integrate((lambda x:np.ones(len(x))), 'y', nmax=1e4,no_protect_names=True)
print("Integral of 1 over a normalized pdf  is ", ret, " and needs to be 1")


# Manual plotting. Note the PDF is NOT renormalized
try:
    xvals = np.linspace(-1,1,100)
    yvals = samplerPrior.pdf['x'](xvals)
    ypvals = samplerPrior.pdf['x'](xvals)/samplerPrior._pdf_norm['x']
    zvals = samplerPrior.cdf['x'](xvals)
    plt.plot(xvals, yvals,label='pdf shape')
    plt.plot(xvals, ypvals,label='pdf')
    plt.plot(xvals, zvals,label='cdf')
    plt.ylim(0,2)
    plt.legend()
    plt.savefig("test_fig_0.png")
except:
    print(" Alternate interfae does not support raw access to priors ")

try:
    # Automated plotting 
    sampler = mcsampler.MCSampler()
    sampler.add_parameter('x', np.vectorize(lambda x: np.exp(-x**2/2)), None, -1,1)
    ourio.plotParameterDistributions('sampler-foridiots-example', sampler)
except:
    print(" Alternate interface does not support alternate plots")

#print "Integral of 1 over this range ", [samplerPrior.llim['y'] ,samplerPrior.rlim['y'] ], " is ", ret, " needs to be 1, because I used a normalized prior"



# Do an MC integral and return the sampling points
# Stop after I reach neff = 100 points (should be fast!)
sig = 0.1
print(" -- Performing integral, stopping after neff = 100 points -- ")
res, var,  neff, dict_return = samplerPrior.integrate(np.vectorize(lambda x: np.exp(-x**2/(2*sig**2))), 'x', nmax=5*1e4, full_output=True,neff=100)
print(" integral answer is ", res,  " with expected error ", np.sqrt(var), ";  compare to ", np.sqrt(2*np.pi)*sig)
print(" note neff is ", neff, "; compare neff^(-1/2) = ", 1/np.sqrt(neff), " to relative predicted and actual errors: ", np.sqrt(var)/res, ", ",  (res - np.sqrt(2*np.pi)*sig)/res)





### mcsamplerEnsemble
print(" --- mcsamplerEnsemble ---- ")
import RIFT.integrators.mcsamplerEnsemble as mcsampler
# Integrate a constant function
samplerPrior = mcsampler.MCSampler()
gmm_dict = {(0,):None}  # note required for this type of integrator, to indicate sampling parameter groups
samplerPrior.add_parameter('x', pdf=np.vectorize(lambda x: 1./(2.5)),  left_limit=-1.5,right_limit=1,adaptive_sampling=True)  # is this correctly renormalized?
# Do an MC integral with this sampler (=the measure specified by the sampler).
def my_func2(x):
    return np.ones(len(x))
ret,var, neff,dict_return = samplerPrior.integrate(my_func2, *['x'], nmax=1e5,verbose=True,gmm_dict=gmm_dict) #,super_verbose=True)
print("Integral of 1 over this range ", [samplerPrior.llim['x'] ,samplerPrior.rlim['x'] ], " is ", ret, " needs to be ", samplerPrior.rlim['x'] -  samplerPrior.llim['x'] ," and (small)")
import sys



sig = 0.1
print(" -- Performing integral of gaussian, stopping after neff = 5000 points -- ")
res, var,  neff, dict_return = samplerPrior.integrate(np.vectorize(lambda x: np.exp(-x**2/(2*sig**2))), 'x', nmax=5*1e4, full_output=True,neff=5000,gmm_dict=gmm_dict)
print(" integral answer is ", res,  " with expected error ", np.sqrt(var), ";  compare to ", np.sqrt(2*np.pi)*sig)
print(" note neff is ", neff, "; compare neff^(-1/2) = ", 1/np.sqrt(neff), " to relative predicted and actual errors: ", np.sqrt(var)/res, ", ",  (res - np.sqrt(2*np.pi)*sig)/res)


sig = 0.1
tempering_exp=0.2
print(" -- repeat test, but with tempering_exp active -- ")
res, var,  neff, dict_return = samplerPrior.integrate(np.vectorize(lambda x: np.exp(-x**2/(2*sig**2))), 'x', nmax=5*1e4, full_output=True,neff=5000,gmm_dict=gmm_dict,tempering_exp=tempering_exp)
print(" integral answer is ", res,  " with expected error ", np.sqrt(var), ";  compare to ", np.sqrt(2*np.pi)*sig)
print(" note neff is ", neff, "; compare neff^(-1/2) = ", 1/np.sqrt(neff), " to relative predicted and actual errors: ", np.sqrt(var)/res, ", ",  (res - np.sqrt(2*np.pi)*sig)/res)


### mcsamplerGPU
print(" --- mcsamplerGPU ---- ")
import RIFT.integrators.mcsamplerGPU as mcsampler
# Integrate a constant function
samplerPrior = mcsampler.MCSampler()
samplerPrior.add_parameter('x', pdf=np.vectorize(lambda x: 1./(2.5)), cdf_inv=None, prior_pdf=np.vectorize(lambda x:1./2.5),left_limit=-1.5,right_limit=1,adaptive_sampling=True)  # is this correctly renormalized?
# Do an MC integral with this sampler (=the measure specified by the sampler).
def my_func2(x):
    return np.ones(len(x))
ret,var, neff,dict_return = samplerPrior.integrate(my_func2, *['x'], nmax=1e5,verbose=True) #,super_verbose=True)
print("Integral of 1 over this range, using a normalized prior, must be 1 ", [samplerPrior.llim['x'] ,samplerPrior.rlim['x'] ], " is ", ret, " needs to be 1")



# PROBLEM
#   - mcsamplerGPU weird normalization factor, ends up getting a magic 'sigma' factor somehow?
samplerPrior.reset_sampling('x')
sig = 0.1
fac=1
n_eff_stop=5000
print(" -- Performing integral of non-normalized gaussian, stopping after neff = {} points -- ".format(n_eff_stop))
res, var,  neff, dict_return = samplerPrior.integrate(np.vectorize(lambda x: fac*np.exp(-x**2/(2*sig**2))), 'x', nmax=5*1e4, full_output=True,neff=n_eff_stop,verbose=True)
print(" integral answer is ", res,  " with expected error ", np.sqrt(var), ";  compare to ", fac* np.sqrt(2*np.pi)*sig)
print(" note neff is ", neff, "; compare neff^(-1/2) = ", 1/np.sqrt(neff), " to relative predicted and actual errors: ", np.sqrt(var)/res, ", ",  (res - np.sqrt(2*np.pi)*sig)/res)


samplerPrior.reset_sampling('x')
#sig = 0.1
tempering_exp=0.2
print(" -- repeat test, but with tempering_exp active -- ")
res, var,  neff, dict_return = samplerPrior.integrate(np.vectorize(lambda x: np.exp(-x**2/(2*sig**2))), 'x', nmax=5*1e4, full_output=True,neff=n_eff_stop,gmm_dict=gmm_dict,tempering_exp=tempering_exp)
print(" integral answer is ", res,  " with expected error ", np.sqrt(var), ";  compare to ", np.sqrt(2*np.pi)*sig)
print(" note neff is ", neff, "; compare neff^(-1/2) = ", 1/np.sqrt(neff), " to relative predicted and actual errors: ", np.sqrt(var)/res, ", ",  (res - np.sqrt(2*np.pi)*sig)/res)
